import argparse
import tarfile
from collections import defaultdict
from typing import Dict

from sacrerouge.data import Metrics, MetricsDict
from sacrerouge.io import JsonlWriter


def load_main_summaries(main_eval_tar_path: str):
    summaries = defaultdict(dict)
    with tarfile.open(main_eval_tar_path, 'r') as tar:
        for member in tar.getmembers():
            if member.name.startswith('mainEval/ROUGE/models/') or member.name.startswith('mainEval/ROUGE/peers/'):
                path = member.name.split('/')
                instance_id = path[-1].split('.')[0].lower()
                summarizer_id = path[-1].split('.')[-1]

                if summarizer_id.isalpha():
                    summarizer_type = 'reference'
                else:
                    summarizer_type = 'peer'

                sentences = tar.extractfile(member).read().decode(errors='replace').splitlines()
                summary = {
                    'summarizer_id': summarizer_id,
                    'summarizer_type': summarizer_type,
                    'text': sentences
                }
                summaries[instance_id][summarizer_id] = summary
    return summaries


def load_update_summaries(update_eval_tar_path: str):
    summaries = defaultdict(lambda: defaultdict(dict))
    with tarfile.open(update_eval_tar_path, 'r') as tar:
        for member in tar.getmembers():
            if member.name.startswith('updateEval/ROUGE/models/') or member.name.startswith('updateEval/ROUGE/peers/'):
                path = member.name.split('/')
                cluster = path[-1].split('.')[0][:-2].lower()
                group = path[-1].split('.')[0][-1]
                summarizer_id = path[-1].split('.')[-1]

                if summarizer_id.isalpha():
                    summarizer_type = 'reference'
                else:
                    summarizer_type = 'peer'

                sentences = tar.extractfile(member).read().decode(errors='replace').splitlines()
                summary = {
                    'summarizer_id': summarizer_id,
                    'summarizer_type': summarizer_type,
                    'text': sentences
                }
                summaries[cluster][summarizer_id][group] = summary
    return summaries


def load_main_rouge_jk_output(eval_tar: str, file_path: str, metrics: Dict[str, Dict[str, Dict[str, MetricsDict]]]):
    jk_metrics = defaultdict(lambda: defaultdict(lambda: defaultdict(lambda: defaultdict(list))))
    with tarfile.open(eval_tar, 'r') as tar:
        lines = tar.extractfile(file_path).read().decode().splitlines()
        for line in lines:
            columns = line.split()
            if len(columns) == 7 and columns[2] == 'Eval':
                summarizer_id = columns[0]
                rouge_metric = columns[1].lower() + '_jk'
                filename = columns[3].split('.')
                instance_id = filename[0].lower()
                summarizer_id = filename[-1]

                recall = float(columns[4][2:]) * 100
                precision = float(columns[5][2:]) * 100
                f1 = float(columns[6][2:]) * 100
                jk_metrics[instance_id][summarizer_id][rouge_metric]['recall'].append(recall)
                jk_metrics[instance_id][summarizer_id][rouge_metric]['precision'].append(precision)
                jk_metrics[instance_id][summarizer_id][rouge_metric]['f1'].append(f1)

        for instance_id in jk_metrics.keys():
            for summarizer_id in jk_metrics[instance_id].keys():
                for rouge_metric in jk_metrics[instance_id][summarizer_id].keys():
                    recalls = jk_metrics[instance_id][summarizer_id][rouge_metric]['recall']
                    precisions = jk_metrics[instance_id][summarizer_id][rouge_metric]['precision']
                    f1s = jk_metrics[instance_id][summarizer_id][rouge_metric]['f1']
                    metrics[instance_id][summarizer_id][rouge_metric] = {
                        'recall': sum(recalls) / len(recalls),
                        'precision': sum(precisions) / len(precisions),
                        'f1': sum(f1s) / len(f1s)
                    }


def load_update_rouge_jk_output(eval_tar: str, file_path: str, metrics: Dict[str, Dict[str, Dict[str, MetricsDict]]]):
    jk_metrics = defaultdict(lambda: defaultdict(lambda: defaultdict(lambda: defaultdict(lambda: defaultdict(list)))))
    with tarfile.open(eval_tar, 'r') as tar:
        lines = tar.extractfile(file_path).read().decode().splitlines()
        for line in lines:
            columns = line.split()
            if len(columns) == 7 and columns[2] == 'Eval':
                summarizer_id = columns[0]
                rouge_metric = columns[1].lower() + '_jk'
                filename = columns[3].split('.')
                instance_id = filename[0].split('-')[0].lower()
                group = filename[0].split('-')[1]
                summarizer_id = filename[-1]

                recall = float(columns[4][2:]) * 100
                precision = float(columns[5][2:]) * 100
                f1 = float(columns[6][2:]) * 100
                jk_metrics[instance_id][group][summarizer_id][rouge_metric]['recall'].append(recall)
                jk_metrics[instance_id][group][summarizer_id][rouge_metric]['precision'].append(precision)
                jk_metrics[instance_id][group][summarizer_id][rouge_metric]['f1'].append(f1)

        for instance_id in jk_metrics.keys():
            for group in ['A', 'B', 'C']:
                for summarizer_id in jk_metrics[instance_id][group].keys():
                    for rouge_metric in jk_metrics[instance_id][group][summarizer_id].keys():
                        recalls = jk_metrics[instance_id][group][summarizer_id][rouge_metric]['recall']
                        precisions = jk_metrics[instance_id][group][summarizer_id][rouge_metric]['precision']
                        f1s = jk_metrics[instance_id][group][summarizer_id][rouge_metric]['f1']
                        metrics[instance_id][group][summarizer_id][rouge_metric] = {
                            'recall': sum(recalls) / len(recalls),
                            'precision': sum(precisions) / len(precisions),
                            'f1': sum(f1s) / len(f1s)
                        }


def load_main_content_table(main_eval_tar_path: str, metrics: Dict[str, Dict[str, Dict[str, MetricsDict]]]):
    with tarfile.open(main_eval_tar_path, 'r') as tar:
        lines = tar.extractfile('mainEval/manual/content.table').read().decode().splitlines()
        for line in lines[6:]:
            columns = line.split()
            instance_id = columns[0].lower()
            summarizer_id = columns[3]
            score = int(columns[4])
            metrics[instance_id][summarizer_id]['content_responsiveness'] = score


def load_update_content_table(update_eval_tar_path: str, metrics: Dict[str, Dict[str, Dict[str, MetricsDict]]]):
    with tarfile.open(update_eval_tar_path, 'r') as tar:
        lines = tar.extractfile('updateEval/Responsiveness/content.table').read().decode().splitlines()
        for line in lines[6:]:
            columns = line.split()
            instance_id = columns[0].split('-')[0].lower()
            group = columns[0].split('-')[1]
            summarizer_id = columns[3]
            score = int(columns[4])
            metrics[instance_id][group][summarizer_id]['content_responsiveness'] = score


def load_main_linguistic_quality_table(main_eval_tar_path: str, metrics: Dict[str, Dict[str, Dict[str, MetricsDict]]]):
    with tarfile.open(main_eval_tar_path, 'r') as tar:
        lines = tar.extractfile('mainEval/manual/linguistic_quality.table').read().decode().splitlines()
        for line in lines[7:]:
            columns = line.split()
            instance_id = columns[0].lower()
            summarizer_id = columns[3]
            question = columns[4]
            score = int(columns[5])
            metrics[instance_id][summarizer_id]['linguistic_quality'][f'Q{question}'] = score


def load_main_pyramid_output(main_pyramid_tar: str, metrics: Dict[str, Dict[str, Dict[str, MetricsDict]]]):
    # This file also has the content and linguistic quality scores, so we probably could
    # have just read in this one file. The other code is already written, so it's not worth changing
    with tarfile.open(main_pyramid_tar, 'r') as tar:
        lines = tar.extractfile('mainPyramidEval/scoring/2007_modified_scores.txt').read().decode().splitlines()
        for line in lines:
            columns = line.split()
            instance_id = columns[0].lower()
            summarizer_id = columns[1]
            metrics[instance_id][summarizer_id]['modified_pyramid_score'] = float(columns[2])
            metrics[instance_id][summarizer_id]['num_scus'] = int(columns[3])
            metrics[instance_id][summarizer_id]['num_repetitions'] = int(columns[4])


def load_update_pyramid_output(update_eval_tar_path: str, metrics: Dict[str, Dict[str, Dict[str, MetricsDict]]]):
    with tarfile.open(update_eval_tar_path, 'r') as tar:
        lines = tar.extractfile('updateEval/Pyramid/scoring/2007_modified_scores.txt').read().decode().splitlines()
        for line in lines:
            columns = line.split()
            instance_id = columns[0].split('-')[0].lower()
            group = columns[0].split('-')[1]
            summarizer_id = columns[1]
            metrics[instance_id][group][summarizer_id]['modified_pyramid_score'] = float(columns[2])
            metrics[instance_id][group][summarizer_id]['num_scus'] = int(columns[3])
            metrics[instance_id][group][summarizer_id]['num_repetitions'] = int(columns[4])


def get_references(summaries, instance_id, summarizer_id, group=None):
    summarizer_ids = list(summaries[instance_id].keys())
    reference_ids = list(filter(lambda sid: sid.isalpha(), summarizer_ids))

    references = []
    for reference_id in reference_ids:
        if summarizer_id == reference_id:
            continue
        if group:
            references.append(summaries[instance_id][reference_id][group])
        else:
            references.append(summaries[instance_id][reference_id])
    return references


def save_main_summaries_metrics(summaries, metrics, output_dir: str):
    with JsonlWriter(f'{output_dir}/task1.summaries.jsonl') as out_summaries:
        with JsonlWriter(f'{output_dir}/task1.metrics.jsonl') as out_metrics:
            for instance_id in sorted(summaries.keys()):
                for summarizer_id in summaries[instance_id].keys():
                    summary = summaries[instance_id][summarizer_id]
                    instance_metrics = metrics[instance_id][summarizer_id]
                    references = get_references(summaries, instance_id, summarizer_id)

                    out_summaries.write({
                        'instance_id': instance_id,
                        'summarizer_id': summarizer_id,
                        'summarizer_type': summary['summarizer_type'],
                        'summary': summary,
                        'references': references
                    })
                    out_metrics.write({
                        'instance_id': instance_id,
                        'summarizer_id': summarizer_id,
                        'summarizer_type': summary['summarizer_type'],
                        'metrics': instance_metrics
                    })


def save_update_summaries_metrics(summaries, metrics, output_dir: str):
    with JsonlWriter(f'{output_dir}/task2.A-B-C.summaries.jsonl') as out_summaries_A_B_C:
        with JsonlWriter(f'{output_dir}/task2.A.summaries.jsonl') as out_summaries_A:
            with JsonlWriter(f'{output_dir}/task2.B.summaries.jsonl') as out_summaries_B:
                with JsonlWriter(f'{output_dir}/task2.C.summaries.jsonl') as out_summaries_C:
                    with JsonlWriter(f'{output_dir}/task2.A-B-C.metrics.jsonl') as out_metrics_A_B_C:
                        with JsonlWriter(f'{output_dir}/task2.A.metrics.jsonl') as out_metrics_A:
                            with JsonlWriter(f'{output_dir}/task2.B.metrics.jsonl') as out_metrics_B:
                                with JsonlWriter(f'{output_dir}/task2.C.metrics.jsonl') as out_metrics_C:
                                    for instance_id in sorted(summaries.keys()):
                                        for summarizer_id in sorted(summaries[instance_id].keys()):
                                            summary_A = summaries[instance_id][summarizer_id]['A']
                                            summary_B = summaries[instance_id][summarizer_id]['B']
                                            summary_C = summaries[instance_id][summarizer_id]['C']

                                            references_A = get_references(summaries, instance_id, summarizer_id, 'A')
                                            references_B = get_references(summaries, instance_id, summarizer_id, 'B')
                                            references_C = get_references(summaries, instance_id, summarizer_id, 'C')

                                            metrics_A = metrics[instance_id]['A'][summarizer_id]
                                            metrics_B = metrics[instance_id]['B'][summarizer_id]
                                            metrics_C = metrics[instance_id]['C'][summarizer_id]

                                            summary_instance_A = {
                                                'instance_id': f'{instance_id}-A',
                                                'summarizer_id': summarizer_id,
                                                'summarizer_type': summary_A['summarizer_type'],
                                                'summary': summary_A,
                                                'references': references_A,
                                            }
                                            summary_instance_B = {
                                                'instance_id': f'{instance_id}-B',
                                                'summarizer_id': summarizer_id,
                                                'summarizer_type': summary_B['summarizer_type'],
                                                'summary': summary_B,
                                                'references': references_B,
                                                'metrics': metrics_B
                                            }
                                            summary_instance_C = {
                                                'instance_id': f'{instance_id}-C',
                                                'summarizer_id': summarizer_id,
                                                'summarizer_type': summary_B['summarizer_type'],
                                                'summary': summary_C,
                                                'references': references_C,
                                                'metrics': metrics_C
                                            }
                                            metric_instance_A = Metrics(f'{instance_id}-A', summarizer_id, summary_A['summarizer_type'], metrics_A)
                                            metric_instance_B = Metrics(f'{instance_id}-B', summarizer_id, summary_B['summarizer_type'], metrics_B)
                                            metric_instance_C = Metrics(f'{instance_id}-C', summarizer_id, summary_C['summarizer_type'], metrics_C)

                                            out_summaries_A_B_C.write(summary_instance_A)
                                            out_summaries_A_B_C.write(summary_instance_B)
                                            out_summaries_A_B_C.write(summary_instance_C)
                                            out_summaries_A.write(summary_instance_A)
                                            out_summaries_B.write(summary_instance_B)
                                            out_summaries_C.write(summary_instance_C)

                                            out_metrics_A_B_C.write(metric_instance_A)
                                            out_metrics_A_B_C.write(metric_instance_B)
                                            out_metrics_A_B_C.write(metric_instance_C)
                                            out_metrics_A.write(metric_instance_A)
                                            out_metrics_B.write(metric_instance_B)
                                            out_metrics_C.write(metric_instance_C)


def setup(data_root: str, output_dir: str):
    main_eval_tar = f'{data_root}/scrapes/duc.nist.gov/past_duc_aquaint/duc2007/results/mainEval.tar.gz'
    update_eval_tar = f'{data_root}/scrapes/duc.nist.gov/past_duc_aquaint/duc2007/results/updateEval.tar.gz'
    main_pyramid_tar = f'{data_root}/scrapes/duc.nist.gov/past_duc_aquaint/duc2007/results/mainPyramidEval.tar.gz'
    main(main_eval_tar, update_eval_tar, main_pyramid_tar, output_dir)


def main(main_eval_tar, update_eval_tar, main_pyramid_tar, output_dir):
    main_summaries = load_main_summaries(main_eval_tar)
    update_summaries = load_update_summaries(update_eval_tar)

    main_metrics = defaultdict(lambda: defaultdict(lambda: defaultdict(MetricsDict)))
    update_metrics = defaultdict(lambda: defaultdict(lambda: defaultdict(MetricsDict)))

    load_main_rouge_jk_output(main_eval_tar, 'mainEval/ROUGE/rougejk.m.out', main_metrics)
    load_update_rouge_jk_output(update_eval_tar, 'updateEval/ROUGE/rougejk.m.out', update_metrics)

    load_main_content_table(main_eval_tar, main_metrics)
    load_update_content_table(update_eval_tar, update_metrics)

    load_main_linguistic_quality_table(main_eval_tar, main_metrics)

    load_main_rouge_jk_output(main_eval_tar, 'mainEval/BE/simplejk.m.hm.out', main_metrics)
    load_update_rouge_jk_output(update_eval_tar, 'updateEval/BE/simplejk.m.hm.out', update_metrics)

    load_main_pyramid_output(main_pyramid_tar, main_metrics)
    load_update_pyramid_output(update_eval_tar, update_metrics)

    save_main_summaries_metrics(main_summaries, main_metrics, output_dir)
    save_update_summaries_metrics(update_summaries, update_metrics, output_dir)


if __name__ == '__main__':
    argp = argparse.ArgumentParser()
    argp.add_argument('main_eval_tar')
    argp.add_argument('update_eval_tar')
    argp.add_argument('main_pyramid_tar')
    argp.add_argument('output_dir')
    args = argp.parse_args()

    main(args.main_eval_tar, args.update_eval_tar, args.main_pyramid_tar, args.output_dir)



import copy
import pytest
from typing import Dict, List, Optional, Union

ValueType = Union['MetricsDict', float, List[float]]


class MetricsDict(dict):
    def __init__(self, initial_dict: Optional[Union['MetricsDict', Dict]] = None) -> None:
        super().__init__()
        if initial_dict:
            initial_dict = copy.deepcopy(initial_dict)
            for key, value in initial_dict.items():
                self[key] = value

    def __setitem__(self, key: str, value: Union[ValueType, Dict]) -> None:
        # If it's a dictionary, we need to convert it to a MetricsDict
        if isinstance(value, dict):
            value_dict = MetricsDict()
            for k, v in value.items():
                value_dict[k] = v
            value = value_dict

        return dict.__setitem__(self, key, value)

    def __getitem__(self, key: str) -> ValueType:
        if not dict.__contains__(self, key):
            dict.__setitem__(self, key, MetricsDict())
        return dict.__getitem__(self, key)

    def flatten_keys(self) -> 'MetricsDict':
        def _recursive_iterate(metrics_dict: MetricsDict, prefix: List[str]):
            for key, value in metrics_dict.items():
                if isinstance(value, MetricsDict):
                    yield from _recursive_iterate(value, prefix + [key])
                else:
                    yield (prefix + [key], value)

        result = MetricsDict()
        for path, value in _recursive_iterate(self, []):
            name = '_'.join(path)
            result[name] = value
        return result

    def average_values(self) -> 'MetricsDict':
        result = MetricsDict()
        for key, value in self.items():
            if isinstance(value, MetricsDict):
                result[key] = value.average_values()
            elif isinstance(value, list):
                result[key] = sum(value) / len(value)
            else:
                result[key] = value
        return result

    def select_metrics(self, metrics: List[str]) -> 'MetricsDict':
        selected = MetricsDict()
        for metric in metrics:
            selected[metric] = self[metric]
        return selected

    def update(self, other: 'MetricsDict') -> None:
        for key, value in other.items():
            if key in self:
                if isinstance(self[key], MetricsDict) and isinstance(value, MetricsDict):
                    self[key].update(value)
                else:
                    self[key] = value
            else:
                self[key] = value

    def _add_to(self, target: 'MetricsDict') -> 'MetricsDict':
        # We created this method so that both __add__ and __radd__ can create
        # an intermediate cumulative MetricsDict and use this method to add to it.
        # Otherwise, it's impossible for __add__ to know if it's adding to the
        # cumulative MetricsDict or the other actual value.
        assert self.keys() == target.keys()
        for key, value in self.items():
            target[key] += value
        return target

    def __add__(self, other: 'MetricsDict') -> 'MetricsDict':
        assert self.keys() == other.keys()
        total = copy.deepcopy(other)
        return self._add_to(total)

    def __radd__(self, other: Union[int, 'MetricsDict']) -> 'MetricsDict':
        if other == 0:
            return copy.deepcopy(self)
        else:
            return self._add_to(other)

    def __truediv__(self, denominator: float) -> 'MetricsDict':
        result = MetricsDict()
        for key, value in self.items():
            result[key] = value / denominator
        return result

    def approx_equal(self, other: 'MetricsDict', rel=None, abs=None):
        if self.keys() != other.keys():
            return False
        for key in self.keys():
            value = self[key]
            other_value = other[key]
            if isinstance(value, MetricsDict) and isinstance(other_value, MetricsDict):
                if not value.approx_equal(other_value, rel=rel, abs=abs):
                    return False
            elif isinstance(value, (int, float)) and isinstance(other_value, (int, float)):
                if not value == pytest.approx(other_value, rel=rel, abs=abs):
                    return False
            elif isinstance(value, list) and isinstance(other_value, list):
                if not value == pytest.approx(other_value, rel=rel, abs=abs):
                    return False
            else:
                # Incompatible types
                return False
        return True



from typing import Any, Dict, List

from sacrerouge.data.pyramid import Pyramid, PyramidAnnotation
from sacrerouge.data.types import DocumentType, ReferenceType, SummaryType


class Field(object):
    def __hash__(self) -> int:
        raise NotImplementedError

    def __eq__(self, other: 'Field') -> bool:
        raise NotImplementedError

    def to_input(self) -> Any:
        """Returns what should be passed as input to the evaluation metric."""
        raise NotImplementedError


class DocumentsField(Field):
    def __init__(self, documents: List[DocumentType]) -> None:
        self.documents = documents

    def __hash__(self) -> int:
        hashes = []
        for document in self.documents:
            if isinstance(document, list):
                hashes.append(hash(tuple(document)))
            else:
                hashes.append(hash(document))
        return hash(tuple(hashes))

    def __eq__(self, other: 'DocumentsField') -> bool:
        return self.documents == other.documents

    def to_input(self) -> List[DocumentType]:
        return self.documents


class PyramidField(Field):
    def __init__(self, pyramid: Pyramid) -> None:
        self.pyramid = pyramid

    def __hash__(self) -> int:
        hashes = []
        hashes.append(hash(self.pyramid.instance_id))
        for summarizer_id in self.pyramid.summarizer_ids:
            hashes.append(hash(summarizer_id))
        return hash(tuple(hashes))

    def __eq__(self, other: 'PyramidField') -> bool:
        return self.pyramid.instance_id == other.pyramid.instance_id and \
               self.pyramid.summarizer_ids == other.pyramid.summarizer_ids

    def to_input(self) -> Pyramid:
        return self.pyramid


class PyramidAnnotationField(Field):
    def __init__(self, annotation: PyramidAnnotation) -> None:
        self.annotation = annotation

    def __hash__(self) -> int:
        return hash(tuple([self.annotation.instance_id, self.annotation.summarizer_id]))

    def __eq__(self, other: 'PyramidAnnotationField') -> bool:
        return self.annotation.instance_id == other.annotation.instance_id and \
               self.annotation.summarizer_id == other.annotation.summarizer_id

    def to_input(self) -> PyramidAnnotation:
        return self.annotation


class ReferencesField(Field):
    def __init__(self, references: List[ReferenceType]) -> None:
        self.references = references

    def __hash__(self) -> int:
        hashes = []
        for reference in self.references:
            if isinstance(reference, list):
                hashes.append(hash(tuple(reference)))
            else:
                hashes.append(hash(reference))
        return hash(tuple(hashes))

    def __eq__(self, other: 'ReferencesField') -> bool:
        return self.references == other.references

    def to_input(self) -> List[ReferenceType]:
        return self.references


class SummaryField(Field):
    def __init__(self, summary: SummaryType) -> None:
        self.summary = summary

    def __hash__(self) -> int:
        if isinstance(self.summary, list):
            return hash(tuple(self.summary))
        return hash(self.summary)

    def __eq__(self, other: 'SummaryField') -> bool:
        return self.summary == other.summary

    def to_input(self) -> SummaryType:
        return self.summary


class Fields(dict):
    def __init__(self, fields: Dict[str, Field]) -> None:
        super().__init__()
        for name, field in fields.items():
            assert isinstance(field, Field)
            self[name] = field

    def select_fields(self, names: List[str]) -> 'Fields':
        return Fields({name: self[name] for name in names})

    def __hash__(self) -> int:
        fields_tuple = tuple([self[name] for name in sorted(self.keys())])
        return hash(fields_tuple)



import argparse
from overrides import overrides

from sacrerouge.datasets.duc_tac.tac2008 import metrics, pyramids, task1
from sacrerouge.commands import DatasetSetupSubcommand


@DatasetSetupSubcommand.register('tac2008')
class TAC2008Subcommand(DatasetSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the TAC 2008 dataset'
        self.parser = parser.add_parser('tac2008', description=description, help=description)
        self.parser.add_argument(
            'data_root',
            type=str,
            help='The path to the root of the repository with the DUC/TAC data (https://github.com/danieldeutsch/duc-tac-data)'
        )
        self.parser.add_argument(
            'output_dir',
            type=str,
            help='The directory where the data should be saved'
        )
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        task1.setup(args.data_root, args.output_dir)
        metrics.setup(args.data_root, args.output_dir)
        pyramids.setup(args.data_root, args.output_dir)



import argparse
import logging
import os
import shutil
from overrides import overrides
from subprocess import Popen, PIPE
from typing import List

from sacrerouge.commands import MetricSetupSubcommand
from sacrerouge.common import DATA_ROOT, TemporaryDirectory
from sacrerouge.data import MetricsDict
from sacrerouge.data.types import ReferenceType, SummaryType
from sacrerouge.metrics import Metric, ReferenceBasedMetric
from sacrerouge.io import JsonlReader, JsonlWriter

logger = logging.getLogger(__name__)


@Metric.register('s3')
class S3(ReferenceBasedMetric):
    def __init__(self,
                 environment_name: str = None,
                 s3_root: str = f'{DATA_ROOT}/metrics/S3',
                 embeddings_file: str = f'{DATA_ROOT}/metrics/S3/deps.words.bz2',
                 model_dir: str = f'{DATA_ROOT}/metrics/S3/models/en',
                 verbose: bool = False):
        super().__init__()
        self.environment_name = environment_name
        self.s3_root = s3_root
        self.embeddings_file = embeddings_file
        self.model_dir = model_dir
        self.verbose = verbose

        if self.environment_name is not None:
            if 'CONDA_INIT' not in os.environ:
                raise Exception('If `environment_name` is not none, environment variable "CONDA_INIT" must be set to the path to "conda.sh"')

    def _flatten_summaries(self, summaries_list: List[List[SummaryType]]) -> List[List[str]]:
        flattened_list = []
        for summaries in summaries_list:
            flattened_list.append([])
            for summary in summaries:
                if isinstance(summary, list):
                    summary = ' '.join(summary)
                flattened_list[-1].append(summary)
        return flattened_list

    def score_multi_all(self,
                        summaries_list: List[List[SummaryType]],
                        references_list: List[List[ReferenceType]],
                        **kwargs) -> List[List[MetricsDict]]:
        summaries_list = self._flatten_summaries(summaries_list)
        references_list = self._flatten_summaries(references_list)

        logger.info(f'Serializing the summaries and references to a file')
        num_summaries = 0
        with TemporaryDirectory() as temp_dir:
            input_file = f'{temp_dir}/input.jsonl'
            output_file = f'{temp_dir}/output.jsonl'
            with JsonlWriter(input_file) as out:
                for summaries, references in zip(summaries_list, references_list):
                    for summary in summaries:
                        out.write({
                            'summary': summary,
                            'references': references
                        })
                        num_summaries += 1
            logger.info(f'Wrote {num_summaries} (summary, references) pairs')

            commands = [f'cd {self.s3_root}/S3']
            if self.environment_name is not None:
                commands.append(f'source {os.environ["CONDA_INIT"]}')
                commands.append(f'conda activate {self.environment_name}')
            commands.append(f'python2.7 run_batch.py {input_file} {output_file} {self.embeddings_file} {self.model_dir}')
            command = ' && '.join(commands)

            logger.info(f'Running command: "{command}"')
            redirect = None if self.verbose else PIPE
            process = Popen(command, stdout=redirect, stderr=redirect, shell=True)
            process.communicate()

            scores = JsonlReader(output_file).read()
            assert len(scores) == num_summaries

            metrics_list = []
            index = 0
            for summaries in summaries_list:
                metrics_list.append([])
                for _ in summaries:
                    metrics_list[-1].append(MetricsDict({
                        's3': {
                            'pyr': scores[index]['pyr'],
                            'resp': scores[index]['resp'],
                        }
                    }))
                    index += 1
            return metrics_list


@MetricSetupSubcommand.register('s3')
class S3SetupSubcommand(MetricSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the S3 metric'
        self.parser = parser.add_parser('s3', description=description, help=description)
        self.parser.add_argument('--force', action='store_true', help='Force setting up the metric again')
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        if args.force and os.path.exists(f'{DATA_ROOT}/metrics/S3'):
            shutil.rmtree(f'{DATA_ROOT}/metrics/S3')

        commands = [
            f'mkdir -p {DATA_ROOT}/metrics',
            f'cd {DATA_ROOT}/metrics',
            f'git clone https://github.com/danieldeutsch/S3',
            f'cd S3',
            f'wget http://u.cs.biu.ac.il/~yogo/data/syntemb/deps.words.bz2'
        ]
        command = ' && '.join(commands)

        process = Popen(command, shell=True)
        process.communicate()

        if process.returncode == 0:
            print('S3 setup success')
        else:
            print('S3 setup failure')



import tarfile
from collections import defaultdict
from io import BytesIO
from nltk.tokenize import sent_tokenize
from typing import Dict, List

from sacrerouge.data import Metrics, MetricsDict
from sacrerouge.io import JsonlWriter


def load_task1_summaries(results_tar: str):
    summaries = defaultdict(lambda: defaultdict(dict))

    with tarfile.open(results_tar, 'r') as tar:
        for inner_tar_path in ['duc2004_results/ROUGE/duc2004.task1.ROUGE.peers.tar.gz', 'duc2004_results/ROUGE/duc2004.task1.ROUGE.models.tar.gz']:
            inner_tar_bytes = tar.extractfile(inner_tar_path).read()
            with tarfile.open(fileobj=BytesIO(inner_tar_bytes)) as inner_tar:
                for member in inner_tar.getmembers():
                    if member.isfile():
                        path = member.name.split('/')
                        parts = path[-1].split('.')

                        filename = parts[5] + '.' + parts[6]
                        summarizer_id = parts[4]
                        if summarizer_id.isalpha():
                            summarizer_type = 'reference'
                        else:
                            summarizer_type = 'peer'

                        text = [inner_tar.extractfile(member).read().decode().strip()]
                        summary = {
                            'summarizer_id': summarizer_id,
                            'summarizer_type': summarizer_type,
                            'text': text
                        }
                        summaries[filename][summarizer_id] = summary
    return summaries


def load_task2_summaries(results_tar: str):
    summaries = defaultdict(lambda: defaultdict(dict))

    with tarfile.open(results_tar, 'r') as tar:
        for inner_tar_path in ['duc2004_results/ROUGE/duc2004.task2.ROUGE.peers.tar.gz', 'duc2004_results/ROUGE/duc2004.task2.ROUGE.models.tar.gz']:
            inner_tar_bytes = tar.extractfile(inner_tar_path).read()
            with tarfile.open(fileobj=BytesIO(inner_tar_bytes)) as inner_tar:
                for member in inner_tar.getmembers():
                    if member.isfile():
                        path = member.name.split('/')
                        parts = path[-1].split('.')

                        instance_id = parts[0].lower()
                        summarizer_id = parts[4]
                        if summarizer_id.isalpha():
                            summarizer_type = 'reference'
                        else:
                            summarizer_type = 'peer'

                        lines = inner_tar.extractfile(member).read().decode().splitlines()
                        sentences = []
                        for line in lines:
                            for sentence in sent_tokenize(line):
                                sentences.append(sentence)

                        summary = {
                            'summarizer_id': summarizer_id,
                            'summarizer_type': summarizer_type,
                            'text': sentences
                        }
                        summaries[instance_id][summarizer_id] = summary
    return summaries


def load_task3_summaries(results_tar: str):
    summaries = defaultdict(lambda: defaultdict(dict))

    with tarfile.open(results_tar, 'r') as tar:
        for inner_tar_path in ['duc2004_results/ROUGE/duc2004.task3.ROUGE.peers.tar.gz', 'duc2004_results/ROUGE/duc2004.task3.ROUGE.models.tar.gz']:
            inner_tar_bytes = tar.extractfile(inner_tar_path).read()
            with tarfile.open(fileobj=BytesIO(inner_tar_bytes)) as inner_tar:
                for member in inner_tar.getmembers():
                    if member.isfile():
                        path = member.name.split('/')
                        parts = path[-1].split('.')

                        filename = parts[5] + '.' + parts[6] + '.' + parts[7]
                        summarizer_id = parts[4]
                        if summarizer_id.isalpha():
                            summarizer_type = 'reference'
                        else:
                            summarizer_type = 'peer'

                        text = [inner_tar.extractfile(member).read().decode().strip()]
                        summary = {
                            'summarizer_id': summarizer_id,
                            'summarizer_type': summarizer_type,
                            'text': text
                        }
                        summaries[filename][summarizer_id] = summary
    return summaries


def load_task4_summaries(results_tar: str):
    summaries = defaultdict(lambda: defaultdict(dict))

    with tarfile.open(results_tar, 'r') as tar:
        for inner_tar_path in ['duc2004_results/ROUGE/duc2004.task4.ROUGE.peers.tar.gz', 'duc2004_results/ROUGE/duc2004.task4.ROUGE.models.tar.gz']:
            inner_tar_bytes = tar.extractfile(inner_tar_path).read()
            with tarfile.open(fileobj=BytesIO(inner_tar_bytes)) as inner_tar:
                for member in inner_tar.getmembers():
                    if member.isfile():
                        path = member.name.split('/')
                        parts = path[-1].split('.')

                        instance_id = parts[0].lower()
                        summarizer_id = parts[4]
                        if summarizer_id.isalpha():
                            summarizer_type = 'reference'
                        else:
                            summarizer_type = 'peer'

                        lines = inner_tar.extractfile(member).read().decode(errors='replace').splitlines()
                        sentences = []
                        for line in lines:
                            for sentence in sent_tokenize(line):
                                sentences.append(sentence)

                        summary = {
                            'summarizer_id': summarizer_id,
                            'summarizer_type': summarizer_type,
                            'text': sentences
                        }
                        summaries[instance_id][summarizer_id] = summary
    return summaries


def load_task5_summaries(results_tar: str):
    summaries = defaultdict(lambda: defaultdict(dict))

    with tarfile.open(results_tar, 'r') as tar:
        for inner_tar_path in ['duc2004_results/ROUGE/duc2004.task5.ROUGE.peers.tar.gz', 'duc2004_results/ROUGE/duc2004.task5.ROUGE.models.tar.gz']:
            inner_tar_bytes = tar.extractfile(inner_tar_path).read()
            with tarfile.open(fileobj=BytesIO(inner_tar_bytes)) as inner_tar:
                for member in inner_tar.getmembers():
                    if member.isfile():
                        path = member.name.split('/')
                        parts = path[-1].split('.')

                        instance_id = parts[0].lower()
                        summarizer_id = parts[4]
                        if summarizer_id.isalpha():
                            summarizer_type = 'reference'
                        else:
                            summarizer_type = 'peer'

                        lines = inner_tar.extractfile(member).read().decode(errors='replace').splitlines()
                        sentences = []
                        for line in lines:
                            for sentence in sent_tokenize(line):
                                sentences.append(sentence)

                        summary = {
                            'summarizer_id': summarizer_id,
                            'summarizer_type': summarizer_type,
                            'text': sentences
                        }
                        summaries[instance_id][summarizer_id] = summary
    return summaries


def load_see_table(results_tar: str, file_path: str, metrics):
    with tarfile.open(results_tar, 'r') as tar:
        lines = tar.extractfile(file_path).read().decode().splitlines()
        for line in lines[34:]:
            columns = line.split()
            assert len(columns) == 23

            instance_id = columns[0].lower()
            summarizer_id = columns[7]
            metrics[instance_id][summarizer_id]['peer_quality']['Q1'] = int(columns[8])
            metrics[instance_id][summarizer_id]['peer_quality']['Q2'] = int(columns[9])
            metrics[instance_id][summarizer_id]['peer_quality']['Q3'] = int(columns[10])
            metrics[instance_id][summarizer_id]['peer_quality']['Q4'] = int(columns[11])
            metrics[instance_id][summarizer_id]['peer_quality']['Q5'] = int(columns[12])
            metrics[instance_id][summarizer_id]['peer_quality']['Q6'] = int(columns[13])
            metrics[instance_id][summarizer_id]['peer_quality']['Q7'] = int(columns[14])
            metrics[instance_id][summarizer_id]['unmarked_units_related_to_subject'] = float(columns[15])
            metrics[instance_id][summarizer_id]['num_peer_units'] = int(columns[16])
            metrics[instance_id][summarizer_id]['num_marked_peer_units'] = int(columns[17])
            metrics[instance_id][summarizer_id]['num_unmarked_peer_units'] = int(columns[18])
            metrics[instance_id][summarizer_id]['num_model_units'] = int(columns[19])
            metrics[instance_id][summarizer_id]['coverage']['mean'] = float(columns[20])
            metrics[instance_id][summarizer_id]['coverage']['median'] = float(columns[21])
            metrics[instance_id][summarizer_id]['coverage']['std'] = float(columns[22])


def load_responsiveness_table(results_tar: str, metrics):
    with tarfile.open(results_tar, 'r') as tar:
        lines = tar.extractfile('duc2004_results/Responsiveness/results.table.R').read().decode().splitlines()
        for line in lines[7:]:
            columns = line.split()
            assert len(columns) == 5

            instance_id = columns[0].lower()
            summarizer_id = columns[3]
            metrics[instance_id][summarizer_id]['responsiveness'] = int(columns[4])


def get_references(summaries, instance_id, summarizer_id):
    summarizer_ids = list(summaries[instance_id].keys())
    reference_ids = list(filter(lambda sid: sid.isalpha(), summarizer_ids))

    references = []
    for reference_id in reference_ids:
        if summarizer_id == reference_id:
            continue
        references.append(summaries[instance_id][reference_id])
    return references


def save_metrics(summaries: Dict[str, Dict[str, List[str]]],
                 metrics: Dict[str, Dict[str, List[int]]],
                 task_name: str,
                 output_dir: str):
    with JsonlWriter(f'{output_dir}/{task_name}.summaries.jsonl') as out_summaries:
        with JsonlWriter(f'{output_dir}/{task_name}.metrics.jsonl') as out_metrics:
            for instance_id in sorted(summaries.keys()):
                for summarizer_id in summaries[instance_id].keys():
                    summary = summaries[instance_id][summarizer_id]
                    instance_metrics = metrics[instance_id][summarizer_id]
                    references = get_references(summaries, instance_id, summarizer_id)

                    out_summaries.write({
                        'instance_id': instance_id,
                        'summarizer_id': summarizer_id,
                        'summarizer_type': summary['summarizer_type'],
                        'summary': summary,
                        'references': references
                    })
                    out_metrics.write(Metrics(instance_id, summarizer_id, summary['summarizer_type'], instance_metrics))


def setup(data_root: str, output_dir: str) -> None:
    results_tar = f'{data_root}/scrapes/duc.nist.gov/past_duc/duc2004/duc2004_results.tgz'
    main(results_tar, output_dir)


def main(results_tar, output_dir):
    task1_summaries = load_task1_summaries(results_tar)
    task2_summaries = load_task2_summaries(results_tar)
    task3_summaries = load_task3_summaries(results_tar)
    task4_summaries = load_task4_summaries(results_tar)
    task5_summaries = load_task5_summaries(results_tar)

    # Tasks 1, 3, and 4 don't have manual judgments.
    # We decided not to parse all of the ROUGE results for the tasks because
    # it does not seem worth the effort since it can be recalculated.
    task2_metrics = defaultdict(lambda: defaultdict(MetricsDict))
    load_see_table(results_tar, 'duc2004_results/SEE/short.results.table2', task2_metrics)

    task5_metrics = defaultdict(lambda: defaultdict(MetricsDict))
    load_see_table(results_tar, 'duc2004_results/SEE/short.results.table5', task5_metrics)
    load_responsiveness_table(results_tar, task5_metrics)

    save_metrics(task2_summaries, task2_metrics, 'task2', output_dir)
    save_metrics(task5_summaries, task5_metrics, 'task5', output_dir)




import argparse
from overrides import overrides

from sacrerouge.datasets.multiling.multiling2011 import metrics, task
from sacrerouge.commands import DatasetSetupSubcommand
from sacrerouge.common.util import download_url_to_file


@DatasetSetupSubcommand.register('multiling2011')
class MultiLing2011Subcommand(DatasetSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the MultiLing 2011 dataset'
        self.parser = parser.add_parser('multiling2011', description=description, help=description)
        self.parser.add_argument(
            'output_dir',
            type=str,
            help='The directory where the data should be saved'
        )
        self.parser.set_defaults(subfunc=self.run)

    def _notify_about_license(self):
        print('The MultiLing2011 dataset is distributed by the MultiLing Pilot '
              'of Text Analysis Conference 2011 under the Create Commons Attribution 3.0 '
              'Unported License. If you use this dataset, please refer to their '
              'website (http://users.iit.demokritos.gr/~ggianna/TAC2011/MultiLing2011.html) '
              'for instructions on proper attribution.')
        print()
        response = input('Continue? [y/n]: ')
        if response.lower() != 'y':
            exit()

    @overrides
    def run(self, args):
        self._notify_about_license()

        # Download the dataset so each individual setup does not need
        # to repeat this. The dataset comes from
        # http://multiling.iit.demokritos.gr/file/view/353/tac-2011-multiling-pilot-dataset-all-files-source-texts-human-and-system-summaries-evaluation-data
        data_path = f'{args.output_dir}/PublishedCorpusFileSet.zip'
        download_url_to_file('http://multiling.iit.demokritos.gr/file/download/353', data_path)

        task.setup(data_path, args.output_dir)
        metrics.setup(data_path, args.output_dir)



import tarfile
from collections import defaultdict
from typing import Set

from sacrerouge.io import JsonlWriter


def load_summaries(eval_tar_path: str):
    summaries = defaultdict(lambda: defaultdict(dict))
    reference_names = set()
    with tarfile.open(eval_tar_path, 'r') as tar:
        for member in tar.getmembers():
            if member.name.startswith('Evaluation_MultiLing2015_MSS/mss-submits/original/') and member.name.endswith('.txt'):
                path = member.name.split('/')
                summarizer_id = path[3] + '-' + path[5]
                language_code = path[4]
                instance_id = path[6][:-4]

                if summarizer_id.startswith('human'):
                    summarizer_type = 'reference'
                    reference_names.add(summarizer_id)
                else:
                    summarizer_type = 'peer'

                text = tar.extractfile(member).read().decode().splitlines()
                text = list(filter(None, [line.strip() for line in text]))
                if len(text) == 0:
                    continue

                summaries[language_code][instance_id][summarizer_id] = {
                    'summarizer_id': summarizer_id,
                    'summarizer_type': summarizer_type,
                    'text': text
                }
    return summaries, reference_names


def get_references(summaries, summarizer_id: str, reference_names: Set[str]):
    references = []
    for name in reference_names:
        if name == summarizer_id:
            continue
        references.append(summaries[name])
    return references


def save_data(summaries, reference_names, output_dir):
    for language in summaries.keys():
        with JsonlWriter(f'{output_dir}/{language}.summaries.jsonl') as out:
            for instance_id in sorted(summaries[language].keys()):
                for summarizer_id in sorted(summaries[language][instance_id].keys()):
                    summary = summaries[language][instance_id][summarizer_id]
                    references = get_references(summaries[language][instance_id], summarizer_id, reference_names)
                    if len(references) == 0:
                        continue
                    out.write({
                        'instance_id': instance_id,
                        'summarizer_id': summarizer_id,
                        'summarizer_type': summary['summarizer_type'],
                        'summary': summary,
                        'references': references
                    })


def setup(eval_tar_path: str, output_dir: str) -> None:
    summaries, reference_names = load_summaries(eval_tar_path)
    save_data(summaries, reference_names, output_dir)


from typing import Any, List, Optional, Tuple

from sacrerouge.common import Registrable
from sacrerouge.data import Jackknifer, MetricsDict
from sacrerouge.data.jackknifers import ReferencesJackknifer
from sacrerouge.data.types import DocumentType, ReferenceType, SummaryType


class Metric(Registrable):
    def __init__(self,
                 required_summary_fields: List[str],
                 required_context_fields: List[str],
                 jackknifer: Optional[Jackknifer] = None) -> None:
        self.required_summary_fields = required_summary_fields
        self.required_context_fields = required_context_fields
        self.jackknifer = jackknifer

    def score(self, *args: List[Any], **kwargs) -> MetricsDict:
        raise NotImplementedError

    def score_multi(self, *args: List[Any], **kwargs) -> List[MetricsDict]:
        raise NotImplementedError

    def score_all(self, *args: List[Any], **kwargs) -> List[MetricsDict]:
        raise NotImplementedError

    def score_multi_all(self, *args: List[Any], **kwargs) -> List[List[MetricsDict]]:
        raise NotImplementedError

    def evaluate(self, *args: List[Any]) -> Tuple[MetricsDict, List[MetricsDict]]:
        raise NotImplementedError

    def aggregate(self, metrics_list: List[MetricsDict]) -> MetricsDict:
        return sum(metrics_list) / len(metrics_list)

    def requires_jackknifing(self) -> bool:
        return self.jackknifer is not None


class SummaryBasedMetric(Metric):
    """
    A `SummaryBasedMetric` is an evaluation metric that requires just a summary and the context fields. Because
    we know exactly how many summary-specific arguments there will be, we can implement a lot of methods in terms
    of `score_multi_all`, which makes it so a metric developer only has to implement one method.
    """
    def score(self, summary: SummaryType, *args: List[Any], **kwargs) -> MetricsDict:
        args = [[arg] for arg in args]
        return self.score_all([summary], *args, **kwargs)[0]

    def score_multi(self, summaries: List[SummaryType], *args: List[Any], **kwargs) -> List[MetricsDict]:
        args = [[arg] for arg in args]
        return self.score_multi_all([summaries], *args, **kwargs)[0]

    def score_all(self, summaries: List[SummaryType], *args: List[List[Any]], **kwargs) -> List[MetricsDict]:
        summaries_list = [[summary] for summary in summaries]
        metrics_lists = self.score_multi_all(summaries_list, *args, **kwargs)
        return [metrics_list[0] for metrics_list in metrics_lists]

    def score_multi_all(self, summaries_list: List[List[SummaryType]], *args: List[List[Any]], **kwargs) -> List[List[MetricsDict]]:
        raise NotImplementedError

    def evaluate(self, summaries: List[SummaryType], *args: List[List[Any]]) -> Tuple[MetricsDict, List[MetricsDict]]:
        micro_metrics_list = self.score_all(summaries, *args)
        macro_metrics = self.aggregate(micro_metrics_list)
        return macro_metrics, micro_metrics_list


class ReferenceBasedMetric(SummaryBasedMetric):
    """
    This is a dummy class that was created to explicitly define the method arguments. It makes the autocomplete
    in some libraries more helpful by supplying argument names like "references" instead of "*args".
    """
    def __init__(self):
        super().__init__(['summary'], ['references'], jackknifer=ReferencesJackknifer())

    def score(self, summary: SummaryType, references: List[ReferenceType], **kwargs) -> MetricsDict:
        return super().score(summary, references, **kwargs)

    def score_multi(self, summaries: List[SummaryType], references: List[ReferenceType], **kwargs) -> List[MetricsDict]:
        return super().score_multi(summaries, references, **kwargs)

    def score_all(self, summaries: List[SummaryType], references_list: List[List[ReferenceType]], **kwargs) -> List[MetricsDict]:
        return super().score_all(summaries, references_list, **kwargs)

    def score_multi_all(self, summaries_list: List[List[SummaryType]], references_list: List[List[ReferenceType]], **kwargs) -> List[List[MetricsDict]]:
        return super().score_multi_all(summaries_list, references_list, **kwargs)

    def evaluate(self, summaries: List[SummaryType], references_list: List[List[ReferenceType]]) -> Tuple[MetricsDict, List[MetricsDict]]:
        return super().evaluate(summaries, references_list)


class DocumentBasedMetric(SummaryBasedMetric):
    """
    This is a dummy class that was created to explicitly define the method arguments. It makes the autocomplete
    in some libraries more helpful by supplying argument names like "documents" instead of "*args".
    """
    def __init__(self):
        super().__init__(['summary'], ['documents'])

    def score(self, summary: SummaryType, documents: List[DocumentType], **kwargs) -> MetricsDict:
        return super().score(summary, documents, **kwargs)

    def score_multi(self, summaries: List[SummaryType], documents: List[DocumentType], **kwargs) -> List[MetricsDict]:
        return super().score_multi(summaries, documents, **kwargs)

    def score_all(self, summaries: List[SummaryType], documents_list: List[List[DocumentType]], **kwargs) -> List[MetricsDict]:
        return super().score_all(summaries, documents_list, **kwargs)

    def score_multi_all(self, summaries_list: List[List[SummaryType]], documents_list: List[List[DocumentType]], **kwargs) -> List[List[MetricsDict]]:
        return super().score_multi_all(summaries_list, documents_list, **kwargs)

    def evaluate(self, summaries: List[SummaryType], documents_list: List[List[DocumentType]]) -> Tuple[MetricsDict, List[MetricsDict]]:
        return super().evaluate(summaries, documents_list)


class ReferenceFreeMetric(SummaryBasedMetric):
    """
    This is a dummy class that was created to explicitly define the method arguments. It makes the autocomplete
    in some libraries more helpful by not supplying the "*args" argument.
    """
    def __init__(self):
        super().__init__(['summary'], [])

    def score(self, summary: SummaryType, **kwargs) -> MetricsDict:
        return super().score(summary, **kwargs)

    def score_multi(self, summaries: List[SummaryType], **kwargs) -> List[MetricsDict]:
        return super().score_multi(summaries, **kwargs)

    def score_all(self, summaries: List[SummaryType], **kwargs) -> List[MetricsDict]:
        return super().score_all(summaries, **kwargs)

    def score_multi_all(self, summaries_list: List[List[SummaryType]], **kwargs) -> List[List[MetricsDict]]:
        return super().score_multi_all(summaries_list, **kwargs)

    def evaluate(self, summaries: List[SummaryType]) -> Tuple[MetricsDict, List[MetricsDict]]:
        return super().evaluate(summaries)



import os
import pytest

from sacrerouge.common.testing import FIXTURES_ROOT
from sacrerouge.common.testing.metric_test_cases import ReferenceBasedMetricTestCase
from sacrerouge.common.testing.util import load_references, load_summaries, sacrerouge_command_exists
from sacrerouge.metrics import Rouge

_duc2004_file_path = 'datasets/duc-tac/duc2004/v1.0/task2.jsonl'
_centroid_file_path = f'{FIXTURES_ROOT}/data/hong2014/centroid.jsonl'
_classy04_file_path = f'{FIXTURES_ROOT}/data/hong2014/classy04.jsonl'
_classy11_file_path = f'{FIXTURES_ROOT}/data/hong2014/classy11.jsonl'
_dpp_file_path = f'{FIXTURES_ROOT}/data/hong2014/dpp.jsonl'
_freq_sum_file_path = f'{FIXTURES_ROOT}/data/hong2014/freq-sum.jsonl'
_greedy_kl_file_path = f'{FIXTURES_ROOT}/data/hong2014/greedy-kl.jsonl'
_icsi_summ_file_path = f'{FIXTURES_ROOT}/data/hong2014/icsi-summ.jsonl'
_lexrank_file_path = f'{FIXTURES_ROOT}/data/hong2014/lexrank.jsonl'
_occams_v_file_path = f'{FIXTURES_ROOT}/data/hong2014/occams-v.jsonl'
_reg_sum_file_path = f'{FIXTURES_ROOT}/data/hong2014/reg-sum.jsonl'
_submodular_file_path = f'{FIXTURES_ROOT}/data/hong2014/submodular.jsonl'
_ts_sum_file_path = f'{FIXTURES_ROOT}/data/hong2014/ts-sum.jsonl'


class TestRouge(ReferenceBasedMetricTestCase):
    @pytest.mark.skipif(not os.path.exists(_duc2004_file_path), reason='DUC 2004 data does not exist')
    @pytest.mark.skipif(not os.path.exists(_centroid_file_path), reason='Hong 2014 data does not exist')
    def test_hong2014(self):
        """
        Tests to ensure that the Rouge scores for the summaries from Hong et al. 2014
        (http://www.lrec-conf.org/proceedings/lrec2014/pdf/1093_Paper.pdf) do not
        change. The hard-coded scores are very close to the scores reported in the paper.
        """
        duc2004 = load_references(_duc2004_file_path)
        centroid = load_summaries(_centroid_file_path)
        classy04 = load_summaries(_classy04_file_path)
        classy11 = load_summaries(_classy11_file_path)
        dpp = load_summaries(_dpp_file_path)
        freq_sum = load_summaries(_freq_sum_file_path)
        greedy_kl = load_summaries(_greedy_kl_file_path)
        icsi_summ = load_summaries(_icsi_summ_file_path)
        lexrank = load_summaries(_lexrank_file_path)
        occams_v = load_summaries(_occams_v_file_path)
        reg_sum = load_summaries(_reg_sum_file_path)
        submodular = load_summaries(_submodular_file_path)
        ts_sum = load_summaries(_ts_sum_file_path)

        rouge = Rouge(max_words=100)

        # Reported: 36.41, 7.97, 1.21
        metrics, _ = rouge.evaluate(centroid, duc2004)
        self.assertAlmostEqual(metrics['rouge-1']['recall'], 36.41, places=2)
        self.assertAlmostEqual(metrics['rouge-2']['recall'], 7.97, places=2)
        self.assertAlmostEqual(metrics['rouge-4']['recall'], 1.21, places=2)

        # Reported: 37.62, 8.96, 1.51
        metrics, _ = rouge.evaluate(classy04, duc2004)
        self.assertAlmostEqual(metrics['rouge-1']['recall'], 37.61, places=2)
        self.assertAlmostEqual(metrics['rouge-2']['recall'], 8.96, places=2)
        self.assertAlmostEqual(metrics['rouge-4']['recall'], 1.51, places=2)

        # Reported: 37.22, 9.20, 1.48
        metrics, _ = rouge.evaluate(classy11, duc2004)
        self.assertAlmostEqual(metrics['rouge-1']['recall'], 37.22, places=2)
        self.assertAlmostEqual(metrics['rouge-2']['recall'], 9.20, places=2)
        self.assertAlmostEqual(metrics['rouge-4']['recall'], 1.48, places=2)

        # Reported: 39.79, 9.62, 1.57
        metrics, _ = rouge.evaluate(dpp, duc2004)
        self.assertAlmostEqual(metrics['rouge-1']['recall'], 39.79, places=2)
        self.assertAlmostEqual(metrics['rouge-2']['recall'], 9.62, places=2)
        self.assertAlmostEqual(metrics['rouge-4']['recall'], 1.57, places=2)

        # Reported: 35.30, 8.11, 1.00
        metrics, _ = rouge.evaluate(freq_sum, duc2004)
        self.assertAlmostEqual(metrics['rouge-1']['recall'], 35.30, places=2)
        self.assertAlmostEqual(metrics['rouge-2']['recall'], 8.11, places=2)
        self.assertAlmostEqual(metrics['rouge-4']['recall'], 1.00, places=2)

        # Reported: 37.98, 8.53, 1.26
        metrics, _ = rouge.evaluate(greedy_kl, duc2004)
        self.assertAlmostEqual(metrics['rouge-1']['recall'], 37.98, places=2)
        self.assertAlmostEqual(metrics['rouge-2']['recall'], 8.53, places=2)
        self.assertAlmostEqual(metrics['rouge-4']['recall'], 1.26, places=2)

        # Reported: 38.41, 9.78, 1.73
        metrics, _ = rouge.evaluate(icsi_summ, duc2004)
        self.assertAlmostEqual(metrics['rouge-1']['recall'], 38.41, places=2)
        self.assertAlmostEqual(metrics['rouge-2']['recall'], 9.78, places=2)
        self.assertAlmostEqual(metrics['rouge-4']['recall'], 1.73, places=2)

        # Reported: 35.95, 7.47, 0.82
        metrics, _ = rouge.evaluate(lexrank, duc2004)
        self.assertAlmostEqual(metrics['rouge-1']['recall'], 35.95, places=2)
        self.assertAlmostEqual(metrics['rouge-2']['recall'], 7.47, places=2)
        self.assertAlmostEqual(metrics['rouge-4']['recall'], 0.82, places=2)

        # Reported: 38.50, 9.76, 1.33
        metrics, _ = rouge.evaluate(occams_v, duc2004)
        self.assertAlmostEqual(metrics['rouge-1']['recall'], 38.50, places=2)
        self.assertAlmostEqual(metrics['rouge-2']['recall'], 9.76, places=2)
        self.assertAlmostEqual(metrics['rouge-4']['recall'], 1.33, places=2)

        # Reported: 38.57, 9.75, 1.60
        metrics, _ = rouge.evaluate(reg_sum, duc2004)
        self.assertAlmostEqual(metrics['rouge-1']['recall'], 38.56, places=2)
        self.assertAlmostEqual(metrics['rouge-2']['recall'], 9.75, places=2)
        self.assertAlmostEqual(metrics['rouge-4']['recall'], 1.60, places=2)

        # Reported: 39.18, 9.35, 1.39
        metrics, _ = rouge.evaluate(submodular, duc2004)
        self.assertAlmostEqual(metrics['rouge-1']['recall'], 39.18, places=2)
        self.assertAlmostEqual(metrics['rouge-2']['recall'], 9.35, places=2)
        self.assertAlmostEqual(metrics['rouge-4']['recall'], 1.39, places=2)

        # Reported: 35.88, 8.15, 1.03
        metrics, _ = rouge.evaluate(ts_sum, duc2004)
        self.assertAlmostEqual(metrics['rouge-1']['recall'], 35.88, places=2)
        self.assertAlmostEqual(metrics['rouge-2']['recall'], 8.14, places=2)
        self.assertAlmostEqual(metrics['rouge-4']['recall'], 1.03, places=2)

    def test_rouge(self):
        # This is a regression test, not necessarily a test for correctness
        metric = Rouge(max_ngram=2, compute_rouge_l=True)
        expected_output = [
            {
                'rouge-1': {'recall': 40.516000000000005, 'precision': 41.699999999999996, 'f1': 41.099000000000004},
                'rouge-2': {'recall': 10.233, 'precision': 10.533, 'f1': 10.381},
                'rouge-l': {'recall': 36.258, 'precision': 37.317, 'f1': 36.78}
            },
            {
                'rouge-1': {'recall': 48.258, 'precision': 47.765, 'f1': 48.010000000000005},
                'rouge-2': {'recall': 19.301, 'precision': 19.103, 'f1': 19.200999999999997},
                'rouge-l': {'recall': 44.774, 'precision': 44.317, 'f1': 44.544}
            },
            {
                'rouge-1': {'recall': 49.416, 'precision': 48.659, 'f1': 49.035000000000004},
                'rouge-2': {'recall': 16.406000000000002, 'precision': 16.154, 'f1': 16.279},
                'rouge-l': {'recall': 45.72, 'precision': 45.019, 'f1': 45.367000000000004}
            },
            {
                'rouge-1': {'recall': 44.466, 'precision': 44.038, 'f1': 44.251000000000005},
                'rouge-2': {'recall': 11.891, 'precision': 11.776, 'f1': 11.833},
                'rouge-l': {'recall': 40.971000000000004, 'precision': 40.577000000000005, 'f1': 40.772999999999996}
            },
            {
                'rouge-1': {'recall': 42.403999999999996, 'precision': 41.473, 'f1': 41.933},
                'rouge-2': {'recall': 10.477, 'precision': 10.245999999999999, 'f1': 10.36},
                'rouge-l': {'recall': 37.649, 'precision': 36.822, 'f1': 37.230999999999995}
            },
            {
                'rouge-1': {'recall': 43.857, 'precision': 43.061, 'f1': 43.455},
                'rouge-2': {'recall': 13.395000000000001, 'precision': 13.150999999999998, 'f1': 13.272},
                'rouge-l': {'recall': 40.555, 'precision': 39.818, 'f1': 40.183}},
            {
                'rouge-1': {'recall': 52.39, 'precision': 51.568999999999996, 'f1': 51.976},
                'rouge-2': {'recall': 20.4, 'precision': 20.079, 'f1': 20.238},
                'rouge-l': {'recall': 47.211, 'precision': 46.471000000000004, 'f1': 46.838}},
            {
                'rouge-1': {'recall': 51.186, 'precision': 51.593999999999994, 'f1': 51.388999999999996},
                'rouge-2': {'recall': 20.238, 'precision': 20.4, 'f1': 20.319000000000003},
                'rouge-l': {'recall': 46.64, 'precision': 47.012, 'f1': 46.825}},
            {
                'rouge-1': {'recall': 38.635999999999996, 'precision': 52.641000000000005, 'f1': 44.564},
                'rouge-2': {'recall': 13.691, 'precision': 18.681, 'f1': 15.801000000000002},
                'rouge-l': {'recall': 35.829, 'precision': 48.815999999999995, 'f1': 41.326}
            },
            {
                'rouge-1': {'recall': 51.73799999999999, 'precision': 51.6, 'f1': 51.669},
                'rouge-2': {'recall': 23.49, 'precision': 23.427, 'f1': 23.458000000000002},
                'rouge-l': {'recall': 49.332, 'precision': 49.2, 'f1': 49.266}
            },
            {
                'rouge-1': {'recall': 48.79, 'precision': 48.016, 'f1': 48.4},
                'rouge-2': {'recall': 21.053, 'precision': 20.717, 'f1': 20.884},
                'rouge-l': {'recall': 47.782000000000004, 'precision': 47.024, 'f1': 47.4}
            },
            {
                'rouge-1': {'recall': 44.711, 'precision': 45.344, 'f1': 45.025},
                'rouge-2': {'recall': 15.03, 'precision': 15.244, 'f1': 15.136},
                'rouge-l': {'recall': 44.112, 'precision': 44.737, 'f1': 44.422}
            }
        ]
        super().assert_expected_output(metric, expected_output)

    def test_rouge_order_invariant(self):
        metric = Rouge(max_words=100)
        self.assert_order_invariant(metric)

    def test_command_exists(self):
        assert sacrerouge_command_exists(['rouge'])

    def test_setup_command_exists(self):
        assert sacrerouge_command_exists(['setup-metric', 'rouge'])


# Modified file from https://github.com/Yale-LILY/SummEval/blob/master/data_processing/pair_data.py
"""
Script for recreating the full model outputs from CNN/DM Story files.
CNN/DM Story files can be downloaded from https://cs.nyu.edu/~kcho/DMQA/
"""
import argparse
import json
import os
from tqdm import tqdm
from glob import glob


def parse_story_file(content):
    """
    Remove article highlights and unnecessary white characters.
    """
    content_raw = content.split("@highlight")[0]
    content = " ".join(filter(None, [x.strip() for x in content_raw.split("\n")]))
    return content

def annotation_pairing(data_annotations, story_files):
    print("Processing file:", data_annotations)
    with open(data_annotations) as fd:
        dataset = [json.loads(line) for line in fd]

    for example in dataset:
        story_path = os.path.join(story_files, example["filepath"])

        with open(story_path) as fd:
            story_content = fd.read()
            example["text"] = parse_story_file(story_content)

    paired_file = data_annotations.replace("aligned", "aligned.paired")
    if os.path.dirname(paired_file):
        os.makedirs(os.path.dirname(paired_file), exist_ok=True)
    with open(paired_file, "w") as fd:
        for example in dataset:
            fd.write(json.dumps(example, ensure_ascii=False) + "\n")


def output_pairing(aligned_data, story_files, model_outputs):
    """
    Walk data sub-directories and recreate examples
    """
    for unpaired_path in glob(f'{aligned_data}/*/aligned/*'):
        filename = os.path.basename(unpaired_path)

        if not (".jsonl" in filename and "aligned" in filename and os.path.isfile(unpaired_path)):
            continue

        print("Processing file:", unpaired_path)
        with open(unpaired_path) as fd:
            dataset = [json.loads(line) for line in fd]

        for example in tqdm(dataset):
            story_path = os.path.join(story_files, example["filepath"])

            with open(story_path) as fd:
                story_content = fd.read()
                example["text"] = parse_story_file(story_content)

        paired_filename = filename.replace("aligned", "aligned.paired")
        paired_path = os.path.join(model_outputs, "paired", paired_filename)
        os.makedirs(os.path.dirname(paired_path), exist_ok=True)
        with open(paired_path, "w") as fd:
            for example in dataset:
                fd.write(json.dumps(example, ensure_ascii=False) + "\n")


def run_pair_data(data_annotations: str = None, model_outputs: str = None, story_files: str = None):
    if not (data_annotations or model_outputs) or not story_files:
            raise RuntimeError("To run script please specify `data_annotations` to pair human annotation data or"
                               "`model_outputs` to pair generated summaries. Story files should be specified in either case.")

    if model_outputs:
        aligned_data = model_outputs # os.path.join(model_outputs, "aligned")

    if data_annotations:
        annotation_pairing(data_annotations, story_files)

    # if model_outputs and story_files:
    #     output_pairing(aligned_data, story_files, model_outputs)

# if __name__ == "__main__":
#     PARSER = argparse.ArgumentParser()
#     PARSER.add_argument("--data_annotations", type=str, help="Path to file human annotations")
#     PARSER.add_argument("--model_outputs", type=str, help="Path to directory holding model data")
#     PARSER.add_argument("--story_files", type=str, help="Path to directory holding CNNDM story files")
#     ARGS = PARSER.parse_args()
#
#
#     if not (ARGS.data_annotations or ARGS.model_outputs) or not ARGS.story_files:
#         raise RuntimeError("To run script please specify `data_annotations` to pair human annotation data or"
#                            "`model_outputs` to pair generated summaries. Story files should be specified in either case.")
#
#     if ARGS.model_outputs:
#         ARGS.aligned_data = os.path.join(ARGS.model_outputs, "aligned")
#
#     if ARGS.data_annotations:
#         annotation_pairing(ARGS)
#
#     if ARGS.model_outputs and ARGS.story_files:
#         output_pairing(ARGS)


import os
import pytest
import unittest

from sacrerouge.commands.correlate import compute_correlation

_metrics_file_path = 'datasets/duc-tac/tac2008/v1.0/task1.A-B.metrics.jsonl'


class TestTAC2008Correlation(unittest.TestCase):
    # Computes the correlations of the TAC 2008 metrics and compares them
    # to the results published in Dang (2008):
    # (https://tac.nist.gov//publications/2008/additional.papers/update_summ_overview08.proceedings.pdf)
    @pytest.mark.skipif(not os.path.exists(_metrics_file_path), reason='TAC 2008 metrics file does not exist')
    def test_dang_2008_table_6(self):
        correlations = compute_correlation(_metrics_file_path, 'overall_responsiveness', 'linguistic_quality', 'reference')
        assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.910, 1e-2)
        assert correlations['system_level']['pearson']['r'] == pytest.approx(0.778, 1e-2)
        assert correlations['system_level']['num_summarizers'] == 8

        correlations = compute_correlation(_metrics_file_path, 'overall_responsiveness', 'linguistic_quality', 'peer')
        assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.750, 1e-2)
        assert correlations['system_level']['pearson']['r'] == pytest.approx(0.763, 1e-2)
        assert correlations['system_level']['num_summarizers'] == 58

        correlations = compute_correlation(_metrics_file_path, 'overall_responsiveness', 'modified_pyramid_score_jk', 'reference')
        assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.455, 1e-2)
        assert correlations['system_level']['pearson']['r'] == pytest.approx(0.637, 1e-2)
        assert correlations['system_level']['num_summarizers'] == 8

        correlations = compute_correlation(_metrics_file_path, 'overall_responsiveness', 'modified_pyramid_score', 'peer')
        assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.941, 1e-2)
        assert correlations['system_level']['pearson']['r'] == pytest.approx(0.950, 1e-2)
        assert correlations['system_level']['num_summarizers'] == 58

    @pytest.mark.skipif(not os.path.exists(_metrics_file_path), reason='TAC 2008 metrics file does not exist')
    def test_dang_2008_table_9(self):
        correlations = compute_correlation(_metrics_file_path, 'modified_pyramid_score_jk', 'rouge-2_jk_recall', 'reference')
        assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.429, 1e-2)
        assert correlations['system_level']['pearson']['r'] == pytest.approx(0.276, 1e-1)
        assert correlations['system_level']['num_summarizers'] == 8

        correlations = compute_correlation(_metrics_file_path, 'modified_pyramid_score_jk', 'rouge-su4_jk_recall', 'reference')
        # assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.595, 1e-2)
        # assert correlations['system_level']['pearson']['r'] == pytest.approx(0.457, 1e-2)
        assert correlations['system_level']['num_summarizers'] == 8

        correlations = compute_correlation(_metrics_file_path, 'modified_pyramid_score_jk', 'rouge-be-hm_jk_recall', 'reference')
        assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.309, 1e-2)
        assert correlations['system_level']['pearson']['r'] == pytest.approx(0.425, 1e-1)
        assert correlations['system_level']['num_summarizers'] == 8

        correlations = compute_correlation(_metrics_file_path, 'modified_pyramid_score', 'rouge-2_recall', 'peer')
        assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.967, 1e-2)
        assert correlations['system_level']['pearson']['r'] == pytest.approx(0.946, 1e-1)
        assert correlations['system_level']['num_summarizers'] == 58

        correlations = compute_correlation(_metrics_file_path, 'modified_pyramid_score', 'rouge-su4_recall', 'peer')
        assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.951, 1e-2)
        assert correlations['system_level']['pearson']['r'] == pytest.approx(0.928, 1e-2)
        assert correlations['system_level']['num_summarizers'] == 58

        correlations = compute_correlation(_metrics_file_path, 'modified_pyramid_score', 'rouge-be-hm_recall', 'peer')
        assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.950, 1e-2)
        assert correlations['system_level']['pearson']['r'] == pytest.approx(0.949, 1e-2)
        assert correlations['system_level']['num_summarizers'] == 58

    @pytest.mark.skipif(not os.path.exists(_metrics_file_path), reason='TAC 2008 metrics file does not exist')
    def test_dang_2008_table_10(self):
        correlations = compute_correlation(_metrics_file_path, 'overall_responsiveness', 'rouge-2_jk_recall', 'reference')
        assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.874, 1e-2)
        assert correlations['system_level']['pearson']['r'] == pytest.approx(0.725, 1e-2)
        assert correlations['system_level']['num_summarizers'] == 8

        correlations = compute_correlation(_metrics_file_path, 'overall_responsiveness', 'rouge-su4_jk_recall', 'reference')
        assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.898, 1e-2)
        assert correlations['system_level']['pearson']['r'] == pytest.approx(0.866, 1e-2)
        assert correlations['system_level']['num_summarizers'] == 8

        correlations = compute_correlation(_metrics_file_path, 'overall_responsiveness', 'rouge-be-hm_jk_recall', 'reference')
        assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.683, 1e-2)
        assert correlations['system_level']['pearson']['r'] == pytest.approx(0.656, 1e-1)
        assert correlations['system_level']['num_summarizers'] == 8

        correlations = compute_correlation(_metrics_file_path, 'overall_responsiveness', 'rouge-2_recall', 'peer')
        assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.920, 1e-2)
        assert correlations['system_level']['pearson']['r'] == pytest.approx(0.894, 1e-2)
        assert correlations['system_level']['num_summarizers'] == 58

        correlations = compute_correlation(_metrics_file_path, 'overall_responsiveness', 'rouge-su4_recall', 'peer')
        assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.909, 1e-2)
        assert correlations['system_level']['pearson']['r'] == pytest.approx(0.874, 1e-2)
        assert correlations['system_level']['num_summarizers'] == 58

        correlations = compute_correlation(_metrics_file_path, 'overall_responsiveness', 'rouge-be-hm_recall', 'peer')
        assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.910, 1e-2)
        assert correlations['system_level']['pearson']['r'] == pytest.approx(0.911, 1e-2)
        assert correlations['system_level']['num_summarizers'] == 58



from spacy.tokens import Token
from typing import List

from sacrerouge.metrics.decomposed_rouge.categorizers import Categorizer


class POSCategorizer(Categorizer):
    def __init__(self, name: str, tags: List[str]):
        super().__init__(name)
        self.tags = set(tags)

    def is_match(self, token1: Token, token2: Token) -> bool:
        return token1.pos_ in self.tags and token2.pos_ in self.tags

    def is_candidate(self, token: Token) -> bool:
        return token.pos_ in self.tags



import unittest

from sacrerouge.common.testing.metric_test_cases import DocumentBasedMetricTestCase
from sacrerouge.common.testing.util import sacrerouge_command_exists
from sacrerouge.metrics import Blanc
from sacrerouge.metrics.blanc import BLANC_INSTALLED


@unittest.skipIf(not BLANC_INSTALLED, '"blanc" not installed')
class TestBlanc(DocumentBasedMetricTestCase):
    def test_command_exists(self):
        assert sacrerouge_command_exists(['blanc'])

    def test_setup_command_exists(self):
        assert sacrerouge_command_exists(['setup-metric', 'blanc'])

    def test_blanc_help_repo(self):
        # Ensures the metric returns the values from a Colab notebook which runs the examples from
        # the Github repo. The notebook fixes the random seed for blanc_tune. Different values are
        # returned for both versions of Blanc than what is reported in the Github repo, but that
        # is probably due to stale documentation.
        #
        # Colab notebook: https://colab.research.google.com/drive/17pJ94L2kCL6QMBMflOm-H0ApBiOUWJ1H?usp=sharing
        metric = Blanc(blanc_type='blanc_help')

        document = "Jack drove his minivan to the bazaar to purchase milk and honey for his large family."
        summary = "Jack bought milk and honey."
        score = metric.score(summary, [document])
        self.assertAlmostEqual(0.2222222222222222, score['blanc_help'], places=4)

        documents = [["Jack drove his minivan to the bazaar to purchase milk and honey for his large family."],
                     ["As Jill started taking a walk in the park, she certainly noticed that the trees were extra green this year."]]
        summaries = ["Jack bought milk and honey.", "Jill saw green trees in the park."]
        scores = metric.score_all(summaries, documents)
        self.assertAlmostEqual(0.2222222222222222, scores[0]['blanc_help'], places=4)
        self.assertAlmostEqual(0.14285714285714285, scores[1]['blanc_help'], places=4)

    def test_blanc_tune_repo(self):
        # See note in `test_blanc_help_repo`
        metric = Blanc(blanc_type='blanc_tune', finetune_mask_evenly=False)

        document = "Jack drove his minivan to the bazaar to purchase milk and honey for his large family."
        summary = "Jack bought milk and honey."
        score = metric.score(summary, [document])
        self.assertAlmostEqual(0.3333333333333333, score['blanc_tune'], places=4)

        metric = Blanc(
            blanc_type='blanc_tune', inference_batch_size=24, finetune_mask_evenly=False, finetune_batch_size=24
        )
        documents = [["Jack drove his minivan to the bazaar to purchase milk and honey for his large family."],
                     ["As Jill started taking a walk in the park, she certainly noticed that the trees were extra green this year."]]
        doc_summaries = [["Jack bought milk and honey.", "Jack drove to the bazaar in a minivan"], ["Jill saw green trees in the park.", "The trees were green."]]
        scores = metric.score_multi_all(doc_summaries, documents)
        self.assertAlmostEqual(0.0, scores[0][0]['blanc_tune'], places=4)
        self.assertAlmostEqual(0.2222222222222222, scores[0][1]['blanc_tune'], places=4)
        self.assertAlmostEqual(0.14285714285714285, scores[1][0]['blanc_tune'], places=4)
        self.assertAlmostEqual(0.07142857142857142, scores[1][1]['blanc_tune'], places=4)



import networkx as nx
from typing import List, Tuple


def _create_graph(matches: List[Tuple[int, int, float]]) -> nx.Graph:
    G = nx.Graph()

    # Add the nodes
    source_nodes = []
    target_notes = []
    source_index_to_node = {}
    target_index_to_node = {}
    source_node_to_index = {}
    target_node_to_index = {}
    count = 1
    for i, j, _ in matches:
        if i not in source_index_to_node:
            source_nodes.append(count)
            source_index_to_node[i] = count
            source_node_to_index[count] = i
            count += 1
        if j not in target_index_to_node:
            target_notes.append(count)
            target_index_to_node[j] = count
            target_node_to_index[count] = j
            count += 1

    G.add_nodes_from(source_nodes, bipartite=0)
    G.add_nodes_from(target_notes, bipartite=1)

    # Add the edges
    for i, j, weight in matches:
        source = source_index_to_node[i]
        target = target_index_to_node[j]
        G.add_edge(source, target, weight=weight)

    return G


def calculate_maximum_matching(matches: List[Tuple[int, int, float]]) -> float:
    if len(matches) == 0:
        return 0.0

    G = _create_graph(matches)

    # Compute the matching and convert the node ids back to the indices
    matching = nx.algorithms.matching.max_weight_matching(G)
    weight = 0
    for i, j in matching:
        weight += G[i][j]['weight']
    return weight


import jsons
from collections import OrderedDict
from jsons import JsonSerializable
from typing import Dict, List, Optional, Union

from sacrerouge.data.metrics_dict import MetricsDict


class Metrics(object):
    def __init__(self,
                 instance_id: str,
                 summarizer_id: str,
                 summarizer_type: str,
                 metrics: Optional[Union[MetricsDict, Dict]] = None) -> None:
        self.instance_id = instance_id
        self.summarizer_id = summarizer_id
        self.summarizer_type = summarizer_type
        self.metrics = MetricsDict(metrics) if metrics else MetricsDict()

    def flatten_keys(self) -> None:
        self.metrics = self.metrics.flatten_keys()

    def average_values(self) -> None:
        self.metrics = self.metrics.average_values()

    def select_metrics(self, metrics: List[str]) -> None:
        self.metrics = self.metrics.select_metrics(metrics)

    def merge(self, other: 'Metrics') -> None:
        if self.instance_id != other.instance_id or \
                self.summarizer_id != other.summarizer_id or \
                self.summarizer_type != other.summarizer_type:
            raise Exception(f'Cannot merge two Metrics if metadata is not the same.')

        for key, value in other.metrics.items():
            self.metrics[key] = value

    def __eq__(self, other: 'Metrics') -> bool:
        if self.instance_id != other.instance_id or \
                self.summarizer_id != other.summarizer_id or \
                self.summarizer_type != other.summarizer_type:
            return False
        return self.metrics == other.metrics

    def __repr__(self) -> str:
        return jsons.dumps(Metrics.serialize(self))

    @staticmethod
    def serialize(metrics: 'Metrics', **kwargs) -> OrderedDict:
        # I think controlling the order of the keys in the serialization
        # aids readability when the json is deserialized. The jsons library
        # will sort the keys if we don't do this
        return OrderedDict({
            'instance_id': metrics.instance_id,
            'summarizer_id': metrics.summarizer_id,
            'summarizer_type': metrics.summarizer_type,
            'metrics': metrics.metrics
        })


JsonSerializable.set_serializer(Metrics.serialize, Metrics)



from collections import Counter
from typing import Dict, List, Tuple

from sacrerouge.common.util import download_file_from_google_drive, download_url_to_file
from sacrerouge.io import JsonlWriter


def _download_documents_and_summaries(output_dir: str, force: bool) -> None:
    # The source files come from the "Raw data, bad retrievals removed" link on https://github.com/Alex-Fabbri/Multi-News
    download_file_from_google_drive('1wHAWDOwOoQWSj7HYpyJ3Aeud8WhhaJ7P', f'{output_dir}/train.src.cleaned', force=force)
    download_file_from_google_drive('1p_u9_jpz3Zbj0EL05QFX6wvJAahmOn6h', f'{output_dir}/val.src.cleaned', force=force)
    download_file_from_google_drive('1-n_6fj-1nM7sWtBSNkQCSfl5Rb3zPVfr', f'{output_dir}/test.src.cleaned', force=force)

    # The target files come from the "Raw data" link
    download_file_from_google_drive('1QVgswwhVTkd3VLCzajK6eVkcrSWEK6kq', f'{output_dir}/train.tgt', force=force)
    download_file_from_google_drive('1Y1lBbBU5Q0aJMqLhYEOdEtTqQ85XnRRM', f'{output_dir}/val.tgt', force=force)
    download_file_from_google_drive('1CX_YcgQ3WwNC1fXBpMfwMXFPCqsd9Lbp', f'{output_dir}/test.tgt', force=force)


def _download_ids(output_dir: str, force: bool) -> None:
    download_url_to_file('https://raw.githubusercontent.com/Alex-Fabbri/Multi-News/master/data/ids/train.id', f'{output_dir}/train.id', force=force)
    download_url_to_file('https://raw.githubusercontent.com/Alex-Fabbri/Multi-News/master/data/ids/val.id', f'{output_dir}/val.id', force=force)
    download_url_to_file('https://raw.githubusercontent.com/Alex-Fabbri/Multi-News/master/data/ids/test.id', f'{output_dir}/test.id', force=force)


def _load_ids(input_file: str) -> List[str]:
    return open(input_file, 'r').read().splitlines()


def _load_documents(input_file: str) -> Tuple[List[List[Dict]], int]:
    documents_list = []
    num_empty = 0
    with open(input_file, 'r') as f:
        for line in f:
            raw_documents = line.strip().split('|||||')
            documents = []
            for raw in raw_documents:
                text = raw.split('NEWLINE_CHAR')
                text = [s.strip() for s in text if len(s.strip()) > 0]
                if len(text) > 0:
                    documents.append({'text': text})
                else:
                    num_empty += 1
            documents_list.append(documents)
    return documents_list, num_empty


def _load_summaries(input_file: str) -> List[Dict]:
    summaries = []
    with open(input_file, 'r') as f:
        for line in f:
            if line.startswith('-'):
                line = line[1:]
            line = line.strip()
            assert len(line) > 0
            summaries.append({'text': line})
    return summaries


def _combine(documents_list: List[List[Dict]], summaries: List[Dict], ids: List[str]) -> List:
    assert len(documents_list) == len(summaries) == len(ids)
    instances = []
    for instance_id, documents, summary in zip(ids, documents_list, summaries):
        instances.append({
            'instance_id': instance_id,
            'documents': documents,
            'summary': summary
        })
    return instances


def _get_stats(instances: List) -> Counter:
    counts = Counter()
    for instance in instances:
        counts[len(instance['documents'])] += 1
    return counts


def _filter_to_having_source_documents(instances: List) -> List:
    filtered = []
    for instance in instances:
        if len(instance['documents']) > 0:
            filtered.append(instance)
    return filtered


def _filter_to_multidoc(instances: List) -> List:
    filtered = []
    for instance in instances:
        if len(instance['documents']) > 1:
            filtered.append(instance)
    return filtered


def _save(instances: List, output_file: str) -> None:
    with JsonlWriter(output_file) as out:
        for instance in instances:
            out.write(instance)


def setup(output_dir: str, force: bool) -> None:
    _download_documents_and_summaries(f'{output_dir}/raw', force)
    _download_ids(f'{output_dir}/raw', force)

    train_ids = _load_ids(f'{output_dir}/raw/train.id')
    valid_ids = _load_ids(f'{output_dir}/raw/val.id')
    test_ids = _load_ids(f'{output_dir}/raw/test.id')

    train_documents, train_empty = _load_documents(f'{output_dir}/raw/train.src.cleaned')
    valid_documents, valid_empty = _load_documents(f'{output_dir}/raw/val.src.cleaned')
    test_documents, test_empty = _load_documents(f'{output_dir}/raw/test.src.cleaned')

    train_summaries = _load_summaries(f'{output_dir}/raw/train.tgt')
    valid_summaries = _load_summaries(f'{output_dir}/raw/val.tgt')
    test_summaries = _load_summaries(f'{output_dir}/raw/test.tgt')

    train = _combine(train_documents, train_summaries, train_ids)
    valid = _combine(valid_documents, valid_summaries, valid_ids)
    test = _combine(test_documents, test_summaries, test_ids)

    print('Number of documents with no text')
    print('Train', train_empty)
    print('Valid', valid_empty)
    print('Test', test_empty)
    print()

    train_stats = _get_stats(train)
    valid_stats = _get_stats(valid)
    test_stats = _get_stats(test)

    train_total = sum(train_stats.values())
    valid_total = sum(valid_stats.values())
    test_total = sum(test_stats.values())

    print('Number of source document counts')
    print('Train', train_stats, train_total)
    print('Valid', valid_stats, valid_total)
    print('Test', test_stats, test_total)
    print()

    train = _filter_to_having_source_documents(train)
    valid = _filter_to_having_source_documents(valid)
    test = _filter_to_having_source_documents(test)

    train_stats = _get_stats(train)
    valid_stats = _get_stats(valid)
    test_stats = _get_stats(test)

    train_total = sum(train_stats.values())
    valid_total = sum(valid_stats.values())
    test_total = sum(test_stats.values())

    print('Number of source document counts (after removing instances with 0 source documents)')
    print('Train', train_stats, train_total)
    print('Valid', valid_stats, valid_total)
    print('Test', test_stats, test_total)
    print()

    _save(train, f'{output_dir}/train.jsonl.gz')
    _save(valid, f'{output_dir}/valid.jsonl.gz')
    _save(test, f'{output_dir}/test.jsonl.gz')

    train_multi = _filter_to_multidoc(train)
    valid_multi = _filter_to_multidoc(valid)
    test_multi = _filter_to_multidoc(test)

    train_stats = _get_stats(train_multi)
    valid_stats = _get_stats(valid_multi)
    test_stats = _get_stats(test_multi)

    train_total = sum(train_stats.values())
    valid_total = sum(valid_stats.values())
    test_total = sum(test_stats.values())

    print('Number of source document counts (after removing instances with 1 source document)')
    print('Train', train_stats, train_total)
    print('Valid', valid_stats, valid_total)
    print('Test', test_stats, test_total)
    print()

    _save(train_multi, f'{output_dir}/train.multi.jsonl.gz')
    _save(valid_multi, f'{output_dir}/valid.multi.jsonl.gz')
    _save(test_multi, f'{output_dir}/test.multi.jsonl.gz')




from spacy.tokens import Token

from sacrerouge.metrics.decomposed_rouge.categorizers import Categorizer


class NPChunkCategorizer(Categorizer):
    def __init__(self):
        super().__init__('np-chunks')

    def is_match(self, token1: Token, token2: Token) -> bool:
        return token1._.is_np is True and token2._.is_np is True

    def is_candidate(self, token: Token) -> bool:
        return token._.is_np is True



import setuptools


# version.py defines the VERSION variable.
# We use exec here so we don't import sacrerouge whilst setting up.
VERSION = {}
with open('sacrerouge/version.py', 'r') as version_file:
    exec(version_file.read(), VERSION)


setuptools.setup(
    name='sacrerouge',
    version=VERSION['VERSION'],
    author='Daniel Deutsch',
    description='An open-source library for summarization evaluation metrics',
    url='https://github.com/danieldeutsch/sacrerouge',
    packages=setuptools.find_packages(),
    entry_points={'console_scripts': ['sacrerouge=sacrerouge.__main__:main']},
    python_requires='>=3.6',
    install_requires=[
        'googledrivedownloader',
        'jsons',
        'lxml',
        'nltk',
        'numpy',
        'overrides==3.1.0',
        'pytest',
        'requests',
        'scipy>=1.5.2',
        'networkx',
        'spacy==2.3.3',
        'matplotlib',
        'sacrebleu==1.5.1',
    ]
)



LANGUAGE_CODES = {
    'arabic': 'ar',
    'chinese': 'zh',
    'czech': 'cs',
    'english': 'en',
    'french': 'fr',
    'greek': 'el',
    'hebrew': 'he',
    'hindi': 'hi',
    'romanian': 'ro',
    'spanish': 'es'
}



import json
import unittest
from subprocess import PIPE, Popen
from typing import Dict, List

from sacrerouge.common import TemporaryDirectory
from sacrerouge.common.testing import FIXTURES_ROOT, MULTILING_SUMMARIES
from sacrerouge.common.testing.util import sacrerouge_command_exists
from sacrerouge.data import Metrics
from sacrerouge.io import JsonlReader

_config_file_path = f'{FIXTURES_ROOT}/configs/evaluate.json'
_numeric_config_file_path = f'{FIXTURES_ROOT}/configs/evaluate-numeric.json'


class TestEvaluate(unittest.TestCase):
    def _check_macro(self, macro: Dict) -> None:
        assert macro['metrics'] == {
            'python-rouge-1': {
                'precision': 47.288299019868255,
                'recall': 46.36412967789611,
                'f1': 46.734012277932806
            },
            'python-rouge-2': {
                'precision': 16.625951207280107,
                'recall': 16.30045341659763,
                'f1': 16.430312150593775
            }
        }

    def _check_micro_list(self, micro_list: List[Metrics]) -> None:
        instances = JsonlReader(MULTILING_SUMMARIES).read()

        assert len(micro_list) == len(instances)
        for micro, instance in zip(micro_list, instances):
            assert micro.instance_id == instance['instance_id']
            assert micro.summarizer_id == instance['summarizer_id']
            assert micro.summarizer_type == instance['summarizer_type']

        assert micro_list[0].metrics == {
            'python-rouge-1': {
                'precision': 41.699867197875164,
                'recall': 40.516129032258064,
                'f1': 41.09947643979057
            },
            'python-rouge-2': {
                'precision': 10.533333333333333,
                'recall': 10.233160621761659,
                'f1': 10.38107752956636
            }
        }
        assert micro_list[11].metrics == {
            'python-rouge-1': {
                'precision': 45.34412955465587,
                'recall': 44.71057884231537,
                'f1': 45.0251256281407
            },
            'python-rouge-2': {
                'precision': 15.24390243902439,
                'recall': 15.030060120240481,
                'f1': 15.136226034308779
            }
        }

    def test_command_exists(self):
        assert sacrerouge_command_exists(['evaluate'])

    def test_evaluate(self):
        # This is a regression test and does not ensure correctness
        with TemporaryDirectory() as temp_dir:
            macro_file = f'{temp_dir}/macro.json'
            micro_file = f'{temp_dir}/micro.jsonl'
            command = [
                'python', '-m', 'sacrerouge', 'evaluate',
                '--config', _config_file_path,
                '--macro-output-json', macro_file,
                '--micro-output-jsonl', micro_file,
                '--silent'
            ]

            process = Popen(command, stdout=PIPE, stderr=PIPE)
            process.communicate()

            macro_metrics = json.load(open(macro_file, 'r'))
            micro_metrics_list = JsonlReader(micro_file, Metrics).read()

            self._check_macro(macro_metrics)
            self._check_micro_list(micro_metrics_list)

    def test_numeric_metric(self):
        with TemporaryDirectory() as temp_dir:
            macro_file = f'{temp_dir}/macro.json'
            micro_file = f'{temp_dir}/micro.jsonl'
            command = [
                'python', '-m', 'sacrerouge', 'evaluate',
                '--config', _numeric_config_file_path,
                '--macro-output-json', macro_file,
                '--micro-output-jsonl', micro_file,
                '--silent'
            ]

            process = Popen(command, stdout=PIPE, stderr=PIPE)
            process.communicate()

            macro_metrics = json.load(open(macro_file, 'r'))
            micro_metrics_list = JsonlReader(micro_file, Metrics).read()

            assert macro_metrics == {'metrics': {'test': 45066}}

            assert len(micro_metrics_list) == 5
            assert micro_metrics_list[0].instance_id == 'D1'
            assert micro_metrics_list[1].instance_id == 'D1'
            assert micro_metrics_list[2].instance_id == 'D1'
            assert micro_metrics_list[3].instance_id == 'D1'
            assert micro_metrics_list[4].instance_id == 'D1'

            assert micro_metrics_list[0].summarizer_id == '1'
            assert micro_metrics_list[1].summarizer_id == '2'
            assert micro_metrics_list[2].summarizer_id == 'A'
            assert micro_metrics_list[3].summarizer_id == 'B'
            assert micro_metrics_list[4].summarizer_id == 'C'

            assert micro_metrics_list[0].summarizer_type == 'peer'
            assert micro_metrics_list[1].summarizer_type == 'peer'
            assert micro_metrics_list[2].summarizer_type == 'reference'
            assert micro_metrics_list[3].summarizer_type == 'reference'
            assert micro_metrics_list[4].summarizer_type == 'reference'

            assert micro_metrics_list[0].metrics == {'test': 1110}  # 1 * 10 + 1 * 100 + 1 * 1000
            assert micro_metrics_list[1].metrics == {'test': 2220}  # 2 * 10 + 2 * 100 + 2 * 1000
            assert micro_metrics_list[2].metrics == {'test': 11000}  # 10 * 100 + 10 * 1000
            assert micro_metrics_list[3].metrics == {'test': 101000}  # 100 * 10 + 100 * 1000
            assert micro_metrics_list[4].metrics == {'test': 110000}  # 1000 * 10 + 10000 * 100


import argparse
from overrides import overrides
from typing import List, Union

from sacrerouge.commands import MetricSetupSubcommand
from sacrerouge.data import MetricsDict
from sacrerouge.data.types import DocumentType, SummaryType
from sacrerouge.metrics import DocumentBasedMetric, Metric


try:
    from blanc import BlancHelp, BlancTune
except ImportError:
    BLANC_INSTALLED = False

    class Blanc(DocumentBasedMetric):
        def __init__(self, *args, **kwargs):
            pass

        def score_multi_all(self, *args, **kwargs):
            raise NotImplementedError('Please install the "blanc" python library library to use Blanc')
else:
    BLANC_INSTALLED = True

    import torch

    @Metric.register('blanc')
    class Blanc(DocumentBasedMetric):
        def __init__(self,
                     blanc_type: str = 'blanc_help',
                     device: str = 'cuda',
                     random_seed: int = 1,
                     **kwargs) -> None:
            super().__init__()
            self.name = blanc_type

            if blanc_type == 'blanc_tune':
                self.blanc = BlancTune(device=device, random_seed=random_seed, **kwargs)
            elif blanc_type == 'blanc_help':
                self.blanc = BlancHelp(device=device, **kwargs)
            else:
                raise Exception(f'Unknown BLANC type: {blanc_type}')

        def _flatten(self, text: Union[DocumentType, SummaryType]) -> str:
            if isinstance(text, list):
                return ' '.join(text)
            return text

        def score_multi_all(self,
                            summaries_list: List[List[SummaryType]],
                            documents_list: List[List[DocumentType]]) -> List[List[MetricsDict]]:
            # For now, the implementation of the BLANC library looks like it only allows for single-document summaries
            documents = []
            for docs in documents_list:
                assert len(docs) == 1, 'BLANC only supports evaluating single-document summaries'
                documents.append(self._flatten(docs[0]))

            summaries_list = [[self._flatten(summary) for summary in summaries] for summaries in summaries_list]

            scores_list = self.blanc.eval_summaries_for_docs(documents, summaries_list)
            metrics_lists = []
            for scores in scores_list:
                metrics_lists.append([])
                for score in scores:
                    metrics_lists[-1].append(MetricsDict({self.name: score}))
            return metrics_lists


@MetricSetupSubcommand.register('blanc')
class BlancSetupSubcommand(MetricSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the BLANC metric'
        self.parser = parser.add_parser('blanc', description=description, help=description)
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        try:
            import blanc
            print('BLANC setup success')
        except ImportError:
            print('Please pip install "blanc" to complete setup')



import os
from pathlib import Path

from sacrerouge.common.checks import ConfigurationError
from sacrerouge.common.params import Params
from sacrerouge.common.from_params import FromParams
from sacrerouge.common.registrable import Registrable
from sacrerouge.common.tempdir import TemporaryDirectory

DATA_ROOT = os.getenv('SACREROUGE_DATA_ROOT', Path.home() / '.sacrerouge')



import json
import pickle
import re
from collections import Counter, defaultdict
from typing import Dict, List, Tuple

from sacrerouge.common.util import download_url_to_file, download_file_from_google_drive
from sacrerouge.data import Metrics, MetricsDict
from sacrerouge.io import JsonlWriter

FILE_IDS = {
    'banditsumm_out': {
        'src': '15q5345IommTx8IjEaXE8stbpHZoeVQWn',
        'ref': '1hqGynlbNeG07AFtZHDE47KZ-9ikvEO5_',
        'out': '17rkbNRMZYQuBFnExsiciBeWbHVJsP0Yy'
    },
    'bart_out': {
        'src': '1B4fo64Jc_rdrpTIuK_LqWdXpHv8T546l',
        'ref': '11DrHN4mI5w3eXdSh_scvrHVdO61d5E0O',
        'out': '1LiIVX-cqk4_Bf7Sw9gFw6SBZgyrF8foX'
    },
    'bottom_up_out': {
        'src': '1E14VYv3XzXp5d7ISefEFbR2v65RnW5TJ',
        'ref': '1mSVEh6ewwjYFYsQc1uTx8CLqzbmBMEX4',
        'out': '1QwgY0EXAkUf7xi08FekRtGLL5FFITgfx'
    },
    'fast_abs_rl_out_rerank': {
        'src': '1mXR4I62uso5SMCBdXSeAAUxF5EN96rwu',
        'ref': '1Piy8hg9Xrh5WVSifmJAz7R11TwvNipQC',
        'out': '1XJyBnQaK7F6ZWLr1f9dmEA8ahBsb-n71'
    },
    'heter_graph_out': {
        'src': '1hLPtWV1puUF4MAEP5k6shbqUJdVl_yTN',
        'ref': '1DQbpzuqSl9JAPj1hJK7ZRrfRbysK_B6r',
        'out': '1mFxDFXAC2cdkKSmeBccOTtcq1Nv0zZRO'
    },
    'matchsumm_out': {
        'src': '1V1NDfucZTOBWSoFRiwXn7nOobbIITWbn',
        'ref': '1tZSetqois0BuS1WUBDRI4qqi2I92RTwu',
        'out': '18q_CbdE5S5pq_GkbLUxvjFwZYyS0yoPX'
    },
    'neusumm_out': {
        'src': '1h_h6lJz6h0mwzfbisgFLBNEdolKz95Py',
        'ref': '1JM9DXmBM2N3yPqO_dNii3Q9NMX5W9p1J',
        'out': '179rmNlBY4Q_cK6b_amqoKk0Dx2PAMjBB',
    },
    'pnbert_out_bert_lstm_pn': {
        'src': '1wMxjAcb9Q3pkiFlk2YgzmG66kXkXLWjy',
        'ref': '1FPtb3gosX0mz__vsYJ454SdUR6oBPK_R',
        'out': '10ppCHPGxv3CPs-Cp7sINq6YdmFp9m3Ci'
    },
    'pnbert_out_bert_lstm_pn_rl': {
        'src': '1OktnpQWcOvQUam7SHCHFm2OHTa-YwFNJ',
        'ref': '1OB7W9WaIiBxj_uKdNuOVRBdic923TnU6',
        'out': '18tC3jVN_ktyTq2_YZ8OQTNZ73XJgpmhv'
    },
    'pnbert_out_bert_tf_pn': {
        'src': '1XeYU68II8bw7FN6XyAbDMaTuD8dCx7Fh',
        'ref': '1Tiv-w0dVP1H3PM6XF-88hM9GSZF1xwBr',
        'out': '1nr11C6C47ixn0nEUS49tzHjVMj9hQSft'
    },
    'pnbert_out_bert_tf_sl': {
        'src': '1p6IKQDhtfxbm-2g6pgKsqpShTS2QhQq7',
        'ref': '1hgQZnNReTruX4Ag2v7xM6lZbQTSi5_wn',
        'out': '1i2GtoOhHhbQGxKf38j0FcNuI-B8F866L'
    },
    'pnbert_out_lstm_pn_rl': {
        'src': '1Zb-DRjnyrH21GsIUQjXU3vRedfMC330_',
        'ref': '1l-7CrF5cXUR4vd6XQhAiEC7vNxHlX3GC',
        'out': '1arRQxV7Emn1ia-2J-IhKEdh8BkUwLLSG'
    },
    'presumm_out_abs': {
        'src': '1IyS2c5ZhpmShAhiiVBV6WxKfpyKPtvcg',
        'ref': '1_BrRh_DklSM8RUFo9qCi1rNOryrKwLdN',
        'out': '120kvLHHnAYGy-QJo3GPWS34kYYMbl99K'
    },
    'presumm_out_ext_abs': {
        'src': '1Eh4n2N3dgJtIBZD4oxj2vptVOy1jC2j0',
        'ref': '1bYIWltkFAYn3d_EcVsRxBClqq1NK5Hck',
        'out': '1Eh4n2N3dgJtIBZD4oxj2vptVOy1jC2j0'
    },
    'presumm_out_trans_abs': {
        'src': '1TwDkwf_M-fck4kQF9VdefG-qPCMrj2yR',
        'ref': '1lmYGJRWdD4aKcQo3GyUttbja_WnnFPoL',
        'out': '13zVlhxyjcsnwnzsdwSGcbgeySusZVQW7'
    },
    'ptr_generator_out_pointer_gen_cov': {
        'src': '13TNp37VKabS8N_2k86PNrymjz7mEp_5C',
        'ref': '1m4O_DH-OsUbJXXWOuheuiVx2LKvgMPZr',
        'out': '1aK6hD21frEzuyW99244aFOlh81VoPNqI'
    },
    'refresh_out': {
        'src': '1FAKShjtVnSIXnXlpgkAjX6TKIkr6bEp4',
        'ref': '1EdZ_Al50jQC-K2R1swuZWK4tlhH_-oTv',
        'out': '1Y5iALSwSs_5NUg-5YIgp1Jb5iSIcMUqv'
    },
    'semsim_out': {
        'src': '1Yy6nV6kLgy4iB62HqTj--QNcu4Oyj9Zc',
        'ref': '19yAw09pLDAeG8vwX8a1mFPtNW1Er6TLR',
        'out': '10E-AYRN79LtSpqBoH1fqit-c2_hVrvPV'
    },
    't5_out_11B': {
        'src': '1H5-Nvelsz3a6h6VzqrOUtU2bAzv9abcD',
        'ref': '1FKBrL-DzR0SaaLTYyDaHwkSq3HGVuNYA',
        'out': '15kmkjQSi0BlvI6W39bugN9IVM5d77ytX'
    },
    't5_out_base': {
        'src': '1VaGXeU5U73uiUdEh8wyb8XDsWQbgjIq6',
        'ref': '1dc-qblHY19smGa8BKK8ZIB8VhFbQBgoW',
        'out': '1upsVycGT91Z94Jdkj34N1A2hw1uvy-Vl'
    },
    't5_out_large': {
        'src': '1sePwJBrQyazWzgAHWRGSIy0R1I0RopJ3',
        'ref': '1jgGFfXa3uAKvH66ujCyto9k45f9s2S4P',
        'out': '1Qiny9zc7COwBq1LtYBtQno5WhhVFcm4q'
    },
    'two_stage_rl_out': {
        'src': '1GV4-9iw9UK09fJAGMZDk9au4Pniic--s',
        'ref': '1lyl9Hn92T9gtF-qtlJtb_obW7uc0cWGu',
        'out': '1voH6p17Z-Rd-_m8Reu86D6TjuwqSG2_H'
    },
    'unilm_out_v1': {
        'src': '1q2WPIUpTQFb8WZEc2hAZNcirIRwDboMx',
        'ref': '1xpjJqNmn_6BFneb5QB9tgpfuAA0cWlak',
        'out': '1AmOhAsmA-kV-PSJSAedg7hSCTJ9_2FVP'
    },
    'unilm_out_v2': {
        'src': '1CgLfz_TNLyAniRNfFKeEZFtDbYFivo_V',
        'ref': '1nuUWeSBWw27hYZlKv9N-ljdB1o5dBmVL',
        'out': '1HyeWRCPjOazOxE3LXx_BmoH6CI2YZkI0'
    },
}

ABSTRACTIVE = {
    'bottom_up_out', 'fast_abs_rl_out_rerank', 'presumm_out_abs', 'presumm_out_ext_abs',
    'presumm_out_trans_abs', 'ptr_generator_out_pointer_gen_cov', 'semsim_out', 't5_out_11B', 't5_out_base',
    't5_out_large', 'two_stage_rl_out', 'unilm_out_v1', 'unilm_out_v2'
}
EXTRACTIVE = {
    'banditsumm_out', 'bart_out', 'heter_graph_out', 'matchsumm_out', 'neusumm_out', 'pnbert_out_bert_lstm_pn',
    'pnbert_out_bert_lstm_pn_rl', 'pnbert_out_bert_tf_pn', 'pnbert_out_bert_tf_sl', 'pnbert_out_lstm_pn_rl', 'refresh_out',
}


def download_raw_data(output_dir: str, force: bool) -> None:
    for summarizer_id, file_ids in FILE_IDS.items():
        for filename, file_id in file_ids.items():
            if file_id is not None:
                download_file_from_google_drive(file_id, f'{output_dir}/raw/{summarizer_id}/{filename}.txt', force=force)


def load_all_data(output_dir: str) -> Tuple[Dict, Dict]:
    abs_data = {}
    ext_data = {}
    for summarizer_id in FILE_IDS.keys():
        if summarizer_id in ABSTRACTIVE:
            data = abs_data
        elif summarizer_id in EXTRACTIVE:
            data = ext_data
        else:
            raise Exception(f'Could not find {summarizer_id}')

        data[summarizer_id] = {
            'src': list(map(lambda line: line.strip(), open(f'{output_dir}/raw/{summarizer_id}/src.txt', 'r').read().splitlines())),
            'ref': list(map(lambda line: line.strip(), open(f'{output_dir}/raw/{summarizer_id}/ref.txt', 'r').read().splitlines())),
            'out': list(map(lambda line: line.strip(), open(f'{output_dir}/raw/{summarizer_id}/out.txt', 'r').read().splitlines()))
        }
        assert len(data[summarizer_id]['src']) == len(data[summarizer_id]['ref']) == len(data[summarizer_id]['out']), \
            f'{summarizer_id} has unequal lines in its src, ref, and out files, likely because Google Drive ' \
            f'began denying requests. Delete the bad files and rerun.'
    return abs_data, ext_data


def get_annotated_summaries(data: Dict):
    summaries = defaultdict(dict)
    for instance_dict in data.values():
        doc_id = str(instance_dict['doc_id'])
        for summarizer_id, summarizer_dict in instance_dict['system_summaries'].items():
            summaries[summarizer_id][summarizer_dict['system_summary'].strip()] = doc_id
    return summaries


def filter_to_scored_summaries(raw_data: Dict, annotated_data: Dict):
    annotated_summaries = get_annotated_summaries(annotated_data)
    filtered = {}
    global_indices_to_keep = None
    for summarizer_id in raw_data.keys():
        indices_to_keep = []
        doc_ids = []
        for i, output in enumerate(raw_data[summarizer_id]['out']):
            if output in annotated_summaries[summarizer_id + '.txt']:
                doc_id = annotated_summaries[summarizer_id + '.txt'][output]
                indices_to_keep.append(i)
                doc_ids.append(doc_id)
        if len(indices_to_keep) != 100:
            raise Exception(f'Could not successfully match the 100 outputs for system {summarizer_id}. This is most '
                            f'likely because Google Drive began denying the download request for all of the system outputs. '
                            f'Please delete the directory: <output-dir>/raw/{summarizer_id} and try again.')

        # Ensures that all of the files are parallel with each other. This ensures that each
        # doc_id corresponds to the same lines in all of the files. We don't rely on this for
        # the annotated summaries, but we do for saving all of the model outputs by grouping all
        # of them by line number
        if global_indices_to_keep is None:
            global_indices_to_keep = indices_to_keep
        assert indices_to_keep == global_indices_to_keep

        filtered[summarizer_id] = {doc_id: {
            'src': raw_data[summarizer_id]['src'][i],
            'ref': raw_data[summarizer_id]['ref'][i],
            'out': raw_data[summarizer_id]['out'][i]
        } for i, doc_id in zip(indices_to_keep, doc_ids)}
    return filtered


def split_into_sentences(text: str) -> List[str]:
    assert text.startswith('<t>'), text
    sentence_regex = '<t> (.+?) </t>'
    sentences = []
    for match in re.findall(sentence_regex, text):
        sentences.append(match)
    return sentences


def convert_to_sacrerouge_instances_and_metrics(annotated_data: Dict, filtered_data: Dict, split: str) -> Tuple[List[Dict], List[Metrics]]:
    instances = []
    metrics_list = []
    for instance_dict in annotated_data.values():
        instance_id = str(instance_dict['doc_id'])

        for summarizer_id, summarizer_dict in instance_dict['system_summaries'].items():
            summarizer_id = summarizer_id[:-4]  # strip .txt
            if split == 'abs' and summarizer_id == 'bart_out':
                # This should not be included in the abstractive models. See the Readme about
                # this dataset
                continue

            document_text = filtered_data[summarizer_id][instance_id]['src']
            if '<t>' in document_text:
                document_text = split_into_sentences(document_text)
                assert len(document_text) > 0
            document = {'text': document_text}

            summary_sentences = split_into_sentences(filtered_data[summarizer_id][instance_id]['out'])
            assert len(summary_sentences) > 0
            summary = {'text': summary_sentences}

            reference_sentences = split_into_sentences(filtered_data[summarizer_id][instance_id]['ref'])
            assert len(reference_sentences) > 0
            reference = {
                'summarizer_id': 'ground-truth',
                'summarizer_type': 'reference',
                'text': reference_sentences
            }

            metrics_dict = MetricsDict({
                'rouge-1': {
                    'precision': float(summarizer_dict['scores']['rouge_1_precision']),
                    'recall': float(summarizer_dict['scores']['rouge_1_recall']),
                    'f1': float(summarizer_dict['scores']['rouge_1_f_score'])
                },
                'rouge-2': {
                    'precision': float(summarizer_dict['scores']['rouge_2_precision']),
                    'recall': float(summarizer_dict['scores']['rouge_2_recall']),
                    'f1': float(summarizer_dict['scores']['rouge_2_f_score'])
                },
                'rouge-l': {
                    'precision': float(summarizer_dict['scores']['rouge_l_precision']),
                    'recall': float(summarizer_dict['scores']['rouge_l_recall']),
                    'f1': float(summarizer_dict['scores']['rouge_l_f_score'])
                },
                'js-2': float(summarizer_dict['scores']['js-2']),
                'MoverScore': float(summarizer_dict['scores']['mover_score']),
                'bertscore': {
                    'precision': float(summarizer_dict['scores']['bert_precision_score']),
                    'recall': float(summarizer_dict['scores']['bert_recall_score']),
                    'f1': float(summarizer_dict['scores']['bert_precision_score']),
                },
                'litepyramid': {
                    'recall': float(summarizer_dict['scores']['litepyramid_recall'])
                }
            })
            instances.append({
                'instance_id': instance_id,
                'summarizer_id': summarizer_id,
                'summarizer_type': 'peer',
                'references': [reference],
                'summary': summary,
                'documents': [document]
            })
            metrics_list.append(Metrics(instance_id, summarizer_id, 'peer', metrics_dict))
    return instances, metrics_list


def save_to_jsonl(data: List, output_file: str) -> None:
    with JsonlWriter(output_file) as out:
        for item in data:
            out.write(item)


def collect_all_outputs(raw_data: Dict) -> List:
    outputs = []
    for summarizer_id, data in raw_data.items():
        src = data['src']
        ref = data['ref']
        pred = data['out']
        for i in range(len(src)):
            summary = pred[i]
            document = src[i]
            reference = ref[i]

            # Some of the data could not be aligned and the summary's content was replaced
            # with this string. We do not include those
            if '### NO MATCH FOUND ###' in summary:
                continue
            if '### IGNORE ###' in summary:
                continue

            if '<t>' in document:
                document = split_into_sentences(document)
            summary = split_into_sentences(summary)
            reference = split_into_sentences(reference)
            if len(summary) == 0:
                # Some of the summaries are "<t>   </t>"
                continue

            # The instance ids in the annotated data do not align with the line numbers in
            # the raw files. We change the instance_id to mark they're from all-summaries to
            # be clear they are different
            outputs.append({
                'instance_id': f'all-summaries-{i}',
                'summarizer_id': summarizer_id,
                'summarizer_type': 'peer',
                'document': {'text': document},
                'summary': {'text': summary},
                'references': [{'text': reference}],
            })
    return outputs


def print_stats(instances: List) -> None:
    summarizer_ids = Counter()
    instance_ids = Counter()

    for instance in instances:
        instance_id = instance['instance_id']
        summarizer_id = instance['summarizer_id']
        summarizer_ids[summarizer_id] += 1
        instance_ids[instance_id] += 1

    print('Num instances per summarizer')
    print(json.dumps(summarizer_ids, indent=2))

    max_count = max(instance_ids.values())
    missing_summaries = {}
    for instance_id, count in instance_ids.items():
        if count != max_count:
            missing_summaries[instance_id] = count

    num_with_max = len(instance_ids) - len(missing_summaries)
    print('Num instances', len(instance_ids))
    print('Num instances with all systems having summaries', num_with_max)
    print('Instnaces with missing summaries')
    print(json.dumps(missing_summaries, indent=2))


def setup(output_dir: str, force: bool = False) -> None:
    download_raw_data(output_dir, force)
    abs_raw_data, ext_raw_data = load_all_data(output_dir)

    download_url_to_file('https://github.com/neulab/REALSumm/raw/master/scores_dicts/abs.pkl', f'{output_dir}/raw/abs.pkl', force=force)
    download_url_to_file('https://github.com/neulab/REALSumm/raw/master/scores_dicts/ext.pkl', f'{output_dir}/raw/ext.pkl', force=force)

    abstractive = pickle.load(open(f'{output_dir}/raw/abs.pkl', 'rb'))
    extractive = pickle.load(open(f'{output_dir}/raw/ext.pkl', 'rb'))

    abs_filtered = filter_to_scored_summaries(abs_raw_data, abstractive)
    ext_filtered = filter_to_scored_summaries(ext_raw_data, extractive)

    abs_instances, abs_metrics_list = convert_to_sacrerouge_instances_and_metrics(abstractive, abs_filtered, 'abs')
    ext_instances, ext_metrics_list = convert_to_sacrerouge_instances_and_metrics(extractive, ext_filtered, 'ext')

    save_to_jsonl(abs_instances, f'{output_dir}/summaries-abs.jsonl')
    save_to_jsonl(ext_instances, f'{output_dir}/summaries-ext.jsonl')
    save_to_jsonl(abs_instances + ext_instances, f'{output_dir}/summaries-mix.jsonl')

    save_to_jsonl(abs_metrics_list, f'{output_dir}/metrics-abs.jsonl')
    save_to_jsonl(ext_metrics_list, f'{output_dir}/metrics-ext.jsonl')
    save_to_jsonl(abs_metrics_list + ext_metrics_list, f'{output_dir}/metrics-mix.jsonl')

    all_abs = collect_all_outputs(abs_raw_data)
    all_ext = collect_all_outputs(ext_raw_data)
    all_mix = collect_all_outputs({**abs_raw_data, **ext_raw_data})

    save_to_jsonl(all_abs, f'{output_dir}/all-summaries-abs.jsonl.gz')
    save_to_jsonl(all_ext, f'{output_dir}/all-summaries-ext.jsonl.gz')
    save_to_jsonl(all_mix, f'{output_dir}/all-summaries-mix.jsonl.gz')

    print_stats(all_mix)



from typing import List, Union

# These are all the same type, but I think it enhances readability
# to have a different type for each one.
DocumentType = Union[str, List[str]]
ReferenceType = Union[str, List[str]]
SummaryType = Union[str, List[str]]



import re
import tarfile
from collections import defaultdict
from nltk.tokenize import sent_tokenize
from typing import List

from sacrerouge.data import Metrics, MetricsDict
from sacrerouge.io import JsonlWriter

PARAGRAPH_SEP_REGEX = re.compile('\r?\n')


def parse_filename(filename: str):
    parts = filename.split('.')
    assert len(parts) == 5
    instance_id = parts[0].split('-')[0].lower()
    group = parts[0].split('-')[1]
    summarizer_id = parts[4]
    return instance_id, group, summarizer_id


def _sent_tokenize(text: str) -> List[str]:
    # Some of the summary files have newlines to separate sentences, others don't.
    # When newlines are present, they are meaningful, so we sentence tokenize within \n-separated text
    sentences = []
    for paragraph in PARAGRAPH_SEP_REGEX.split(text):
        paragraph = paragraph.strip()
        if paragraph:
            for sentence in sent_tokenize(paragraph):
                sentences.append(sentence)
    return sentences


def load_summaries(eval_tar: str):
    summaries = defaultdict(lambda: defaultdict(dict))
    with tarfile.open(eval_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile() and (member.name.startswith('GuidedSumm2011_eval/ROUGE/models/') or member.name.startswith('GuidedSumm2011_eval/ROUGE/peers/')):
                path = member.name.split('/')
                filename = path[-1]
                instance_id, group, summarizer_id = parse_filename(filename)

                if summarizer_id.isalpha():
                    summarizer_type = 'reference'
                else:
                    summarizer_type = 'peer'

                text = tar.extractfile(member).read().decode(errors='replace').strip()
                sentences = _sent_tokenize(text)
                summary = {
                    'summarizer_id': summarizer_id,
                    'summarizer_type': summarizer_type,
                    'text': sentences
                }
                summaries[instance_id][summarizer_id][group] = summary
    return summaries


def load_manual_judgments(eval_tar: str, metrics):
    with tarfile.open(eval_tar, 'r') as tar:
        for filename in ['manual.model.A', 'manual.model.B']:
            lines = tar.extractfile(f'GuidedSumm2011_eval/manual/{filename}').read().decode().splitlines()
            for line in lines:
                columns = line.split()
                instance_id = columns[0].split('-')[0].lower()
                group = columns[0].split('-')[1]
                summarizer_id = columns[1]
                metrics[instance_id][group][summarizer_id]['num_scus_jk'] = int(columns[2])
                metrics[instance_id][group][summarizer_id]['modified_pyramid_score_jk'] = float(columns[5])
                metrics[instance_id][group][summarizer_id]['linguistic_quality'] = int(columns[6])
                metrics[instance_id][group][summarizer_id]['overall_responsiveness'] = int(columns[7])

        for filename in ['manual.peer.A', 'manual.peer.B']:
            lines = tar.extractfile(f'GuidedSumm2011_eval/manual/{filename}').read().decode().splitlines()
            for line in lines:
                columns = line.split()
                instance_id = columns[0].split('-')[0].lower()
                group = columns[0].split('-')[1]
                summarizer_id = columns[1]
                metrics[instance_id][group][summarizer_id]['modified_pyramid_score'] = float(columns[2])
                metrics[instance_id][group][summarizer_id]['num_scus'] = int(columns[3])
                metrics[instance_id][group][summarizer_id]['num_repetitions'] = int(columns[4])
                metrics[instance_id][group][summarizer_id]['modified_pyramid_score_jk'] = float(columns[7])
                metrics[instance_id][group][summarizer_id]['linguistic_quality'] = int(columns[8])
                metrics[instance_id][group][summarizer_id]['overall_responsiveness'] = int(columns[9])


def load_rouge_output(eval_tar: str, file_path1: str, file_path2: str, metrics):
    with tarfile.open(eval_tar, 'r') as tar:
        for file_path in [file_path1, file_path2]:
            lines = tar.extractfile(file_path).read().decode().splitlines()
            for line in lines:
                columns = line.split()
                if len(columns) == 7 and columns[2] == 'Eval':
                    summarizer_id = columns[0]
                    rouge_metric = columns[1].lower()
                    instance_id, group, summarizer_id = parse_filename(columns[3])
                    recall = float(columns[4][2:]) * 100
                    precision = float(columns[5][2:]) * 100
                    f1 = float(columns[6][2:]) * 100
                    metrics[instance_id][group][summarizer_id][rouge_metric] = {
                        'recall': recall,
                        'precision': precision,
                        'f1': f1
                    }


def load_rouge_jk_output(eval_tar: str, file_path1: str, file_path2: str, metrics):
    jk_metrics = defaultdict(lambda: defaultdict(lambda: defaultdict(lambda: defaultdict(lambda: defaultdict(list)))))
    with tarfile.open(eval_tar, 'r') as tar:
        for file_path in [file_path1, file_path2]:
            lines = tar.extractfile(file_path).read().decode().splitlines()
            for line in lines:
                columns = line.split()
                if len(columns) == 7 and columns[2] == 'Eval':
                    summarizer_id = columns[0]
                    rouge_metric = columns[1].lower() + '_jk'
                    instance_id, group, summarizer_id = parse_filename(columns[3])

                    recall = float(columns[4][2:]) * 100
                    precision = float(columns[5][2:]) * 100
                    f1 = float(columns[6][2:]) * 100
                    jk_metrics[instance_id][group][summarizer_id][rouge_metric]['recall'].append(recall)
                    jk_metrics[instance_id][group][summarizer_id][rouge_metric]['precision'].append(precision)
                    jk_metrics[instance_id][group][summarizer_id][rouge_metric]['f1'].append(f1)

            for instance_id in jk_metrics.keys():
                for group in ['A', 'B']:
                    for summarizer_id in jk_metrics[instance_id][group].keys():
                        for rouge_metric in jk_metrics[instance_id][group][summarizer_id].keys():
                            recalls = jk_metrics[instance_id][group][summarizer_id][rouge_metric]['recall']
                            precisions = jk_metrics[instance_id][group][summarizer_id][rouge_metric]['precision']
                            f1s = jk_metrics[instance_id][group][summarizer_id][rouge_metric]['f1']
                            metrics[instance_id][group][summarizer_id][rouge_metric] = {
                                'recall': sum(recalls) / len(recalls),
                                'precision': sum(precisions) / len(precisions),
                                'f1': sum(f1s) / len(f1s)
                            }


def load_aesop_metrics(aesop_tar: str, metrics):
    with tarfile.open(aesop_tar, 'r') as tar:
        for member in tar.getmembers():
            for group, filename in zip(['A', 'B'], ['AESOP2011_eval_V1.2/data/aesop_allpeers_A', 'AESOP2011_eval_V1.2/data/aesop_allpeers_B']):
                lines = tar.extractfile(filename).read().decode().splitlines()
                for line in lines[1:]:
                    columns = line.split()
                    summarizer_id = columns[0][1:]
                    instance_number = int(columns[1][1:])
                    instance_id = 'd' + str(1100 + instance_number)
                    for i, value in enumerate(columns[2:]):
                        if value != 'NaN':
                            metrics[instance_id][group][summarizer_id]['aesop'][str(i + 1)] = float(value)


def get_references(summaries, instance_id, summarizer_id, group):
    summarizer_ids = list(summaries[instance_id].keys())
    reference_ids = list(filter(lambda sid: sid.isalpha(), summarizer_ids))

    references = []
    for reference_id in reference_ids:
        if summarizer_id == reference_id:
            continue
        references.append(summaries[instance_id][reference_id][group])
    return references


def save_summaries_and_metrics(summaries, metrics, output_dir: str):
    with JsonlWriter(f'{output_dir}/task1.A-B.summaries.jsonl') as out_summaries_A_B:
        with JsonlWriter(f'{output_dir}/task1.A.summaries.jsonl') as out_summaries_A:
            with JsonlWriter(f'{output_dir}/task1.B.summaries.jsonl') as out_summaries_B:
                with JsonlWriter(f'{output_dir}/task1.A-B.metrics.jsonl') as out_metrics_A_B:
                    with JsonlWriter(f'{output_dir}/task1.A.metrics.jsonl') as out_metrics_A:
                        with JsonlWriter(f'{output_dir}/task1.B.metrics.jsonl') as out_metrics_B:
                            for instance_id in sorted(summaries.keys()):
                                for summarizer_id in sorted(summaries[instance_id].keys()):
                                    summary_A = summaries[instance_id][summarizer_id]['A']
                                    summary_B = summaries[instance_id][summarizer_id]['B']

                                    references_A = get_references(summaries, instance_id, summarizer_id, 'A')
                                    references_B = get_references(summaries, instance_id, summarizer_id, 'B')

                                    metrics_A = metrics[instance_id]['A'][summarizer_id]
                                    metrics_B = metrics[instance_id]['B'][summarizer_id]

                                    summary_instance_A = {
                                        'instance_id': f'{instance_id}-A',
                                        'summarizer_id': summarizer_id,
                                        'summarizer_type': summary_A['summarizer_type'],
                                        'summary': summary_A,
                                        'references': references_A,
                                    }
                                    summary_instance_B = {
                                        'instance_id': f'{instance_id}-B',
                                        'summarizer_id': summarizer_id,
                                        'summarizer_type': summary_B['summarizer_type'],
                                        'summary': summary_B,
                                        'references': references_B,
                                        'metrics': metrics_B
                                    }
                                    metric_instance_A = Metrics(f'{instance_id}-A', summarizer_id, summary_A['summarizer_type'], metrics_A)
                                    metric_instance_B = Metrics(f'{instance_id}-B', summarizer_id, summary_B['summarizer_type'], metrics_B)

                                    out_summaries_A_B.write(summary_instance_A)
                                    out_summaries_A_B.write(summary_instance_B)
                                    out_summaries_A.write(summary_instance_A)
                                    out_summaries_B.write(summary_instance_B)

                                    out_metrics_A_B.write(metric_instance_A)
                                    out_metrics_A_B.write(metric_instance_B)
                                    out_metrics_A.write(metric_instance_A)
                                    out_metrics_B.write(metric_instance_B)


def setup(data_root: str, output_dir: str):
    eval_tar = f'{data_root}/scrapes/tac.nist.gov/protected/past/2011/GuidedSumm2011_eval.tgz'
    aesop_tar = f'{data_root}/scrapes/tac.nist.gov/protected/past/2011/AESOP2011_eval_V1.2.tgz'
    main(eval_tar, aesop_tar, output_dir)


def main(eval_tar, aesop_tar, output_dir):
    summaries = load_summaries(eval_tar)

    metrics = defaultdict(lambda: defaultdict(lambda: defaultdict(MetricsDict)))
    load_manual_judgments(eval_tar, metrics)
    load_rouge_output(eval_tar, 'GuidedSumm2011_eval/ROUGE/rouge_A.m.out', 'GuidedSumm2011_eval/ROUGE/rouge_B.m.out', metrics)
    load_rouge_jk_output(eval_tar, 'GuidedSumm2011_eval/ROUGE/rougejk_A.m.out', 'GuidedSumm2011_eval/ROUGE/rougejk_B.m.out', metrics)
    load_rouge_output(eval_tar, 'GuidedSumm2011_eval/BE/simple_A.m.hm.out', 'GuidedSumm2011_eval/BE/simple_B.m.hm.out', metrics)
    load_rouge_jk_output(eval_tar, 'GuidedSumm2011_eval/BE/simplejk_A.m.hm.out', 'GuidedSumm2011_eval/BE/simplejk_B.m.hm.out', metrics)
    load_aesop_metrics(aesop_tar, metrics)

    save_summaries_and_metrics(summaries, metrics, output_dir)



import os
import pytest
import unittest

from sacrerouge.commands.correlate import aggregate_metrics
from sacrerouge.data import Metrics
from sacrerouge.io import JsonlReader

_metrics_A_file_path = 'datasets/duc-tac/tac2011/v1.0/task1.A.metrics.jsonl'
_metrics_B_file_path = 'datasets/duc-tac/tac2011/v1.0/task1.B.metrics.jsonl'


class TestTAC2011SystemLevel(unittest.TestCase):
    @pytest.mark.skipif(not os.path.exists(_metrics_A_file_path), reason='TAC 2011-A metrics file does not exist')
    def test_system_level_A(self):
        summary_level_metrics = JsonlReader(_metrics_A_file_path, Metrics).read()
        system_level_metrics = aggregate_metrics(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # ROUGE/rouge2_A.m.avg
        assert system_level_metrics['43']['rouge-2']['recall'] == pytest.approx(13.440, 1e-2)
        assert system_level_metrics['17']['rouge-2']['recall'] == pytest.approx(12.994, 1e-2)
        assert system_level_metrics['25']['rouge-2']['recall'] == pytest.approx(12.821, 1e-2)

        # ROUGE/rouge2_A.jk.m.avg
        assert system_level_metrics['D']['rouge-2_jk']['recall'] == pytest.approx(12.820, 1e-2)
        assert system_level_metrics['43']['rouge-2_jk']['recall'] == pytest.approx(13.447, 1e-2)
        assert system_level_metrics['17']['rouge-2_jk']['recall'] == pytest.approx(12.993, 1e-2)

        # ROUGE/rougeSU4_A.m.avg
        assert system_level_metrics['43']['rouge-su4']['recall'] == pytest.approx(16.519, 1e-2)
        assert system_level_metrics['17']['rouge-su4']['recall'] == pytest.approx(15.984, 1e-2)
        assert system_level_metrics['24']['rouge-su4']['recall'] == pytest.approx(15.975, 1e-2)

        # ROUGE/rougeSU4_A.jk.m.avg
        assert system_level_metrics['D']['rouge-su4_jk']['recall'] == pytest.approx(16.412, 1e-2)
        assert system_level_metrics['A']['rouge-su4_jk']['recall'] == pytest.approx(16.118, 1e-2)
        assert system_level_metrics['43']['rouge-su4_jk']['recall'] == pytest.approx(16.519, 1e-2)

        # manual/manual.model.A.avg
        assert system_level_metrics['A']['num_scus_jk'] == pytest.approx(10.227, 1e-2)
        assert system_level_metrics['B']['num_scus_jk'] == pytest.approx(9.773, 1e-2)
        assert system_level_metrics['C']['num_scus_jk'] == pytest.approx(9.818, 1e-2)

        assert system_level_metrics['A']['modified_pyramid_score_jk'] == pytest.approx(0.771, 1e-2)
        assert system_level_metrics['B']['modified_pyramid_score_jk'] == pytest.approx(0.781, 1e-2)
        assert system_level_metrics['C']['modified_pyramid_score_jk'] == pytest.approx(0.752, 1e-2)

        assert system_level_metrics['A']['linguistic_quality'] == pytest.approx(4.864, 1e-2)
        assert system_level_metrics['B']['linguistic_quality'] == pytest.approx(4.818, 1e-2)
        assert system_level_metrics['C']['linguistic_quality'] == pytest.approx(5.000, 1e-2)

        assert system_level_metrics['A']['overall_responsiveness'] == pytest.approx(4.818, 1e-2)
        assert system_level_metrics['B']['overall_responsiveness'] == pytest.approx(4.727, 1e-2)
        assert system_level_metrics['C']['overall_responsiveness'] == pytest.approx(4.955, 1e-2)

        # manual/manual.peer.A.avg
        assert system_level_metrics['1']['modified_pyramid_score'] == pytest.approx(0.304, 1e-2)
        assert system_level_metrics['2']['modified_pyramid_score'] == pytest.approx(0.362, 1e-2)
        assert system_level_metrics['3']['modified_pyramid_score'] == pytest.approx(0.439, 1e-2)

        assert system_level_metrics['1']['num_scus'] == pytest.approx(3.909, 1e-2)
        assert system_level_metrics['2']['num_scus'] == pytest.approx(4.614, 1e-2)
        assert system_level_metrics['3']['num_scus'] == pytest.approx(5.750, 1e-2)

        assert system_level_metrics['1']['num_repetitions'] == pytest.approx(0.455, 1e-2)
        assert system_level_metrics['2']['num_repetitions'] == pytest.approx(1.432, 1e-2)
        assert system_level_metrics['3']['num_repetitions'] == pytest.approx(1.409, 1e-2)

        assert system_level_metrics['1']['modified_pyramid_score_jk'] == pytest.approx(0.300, 1e-2)
        assert system_level_metrics['2']['modified_pyramid_score_jk'] == pytest.approx(0.358, 1e-2)
        assert system_level_metrics['3']['modified_pyramid_score_jk'] == pytest.approx(0.433, 1e-2)

        assert system_level_metrics['1']['linguistic_quality'] == pytest.approx(3.205, 1e-2)
        assert system_level_metrics['2']['linguistic_quality'] == pytest.approx(2.818, 1e-2)
        assert system_level_metrics['3']['linguistic_quality'] == pytest.approx(2.705, 1e-2)

        assert system_level_metrics['1']['overall_responsiveness'] == pytest.approx(2.500, 1e-2)
        assert system_level_metrics['2']['overall_responsiveness'] == pytest.approx(2.841, 1e-2)
        assert system_level_metrics['3']['overall_responsiveness'] == pytest.approx(3.045, 1e-2)

        # BE/simple_A.m.hm.avg
        assert system_level_metrics['43']['rouge-be-hm']['recall'] == pytest.approx(8.565, 1e-2)
        assert system_level_metrics['17']['rouge-be-hm']['recall'] == pytest.approx(8.153, 1e-2)
        assert system_level_metrics['25']['rouge-be-hm']['recall'] == pytest.approx(8.012, 1e-2)

        # BE/simplejk_A.m.hm.avg
        assert system_level_metrics['D']['rouge-be-hm_jk']['recall'] == pytest.approx(9.085, 1e-2)
        assert system_level_metrics['E']['rouge-be-hm_jk']['recall'] == pytest.approx(8.628, 1e-2)
        assert system_level_metrics['43']['rouge-be-hm_jk']['recall'] == pytest.approx(8.553, 1e-2)

        # aesop_allpeers_A
        assert system_level_metrics['A']['aesop']['1'] == pytest.approx(0.1191786364, 1e-2)
        assert system_level_metrics['C']['aesop']['8'] == pytest.approx(3.853212409, 1e-2)
        assert system_level_metrics['4']['aesop']['13'] == pytest.approx(0.4008335416, 1e-2)

    @pytest.mark.skipif(not os.path.exists(_metrics_B_file_path), reason='TAC 2011-B metrics file does not exist')
    def test_system_level_B(self):
        summary_level_metrics = JsonlReader(_metrics_B_file_path, Metrics).read()
        system_level_metrics = aggregate_metrics(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # ROUGE/rouge2_B.m.avg
        assert system_level_metrics['43']['rouge-2']['recall'] == pytest.approx(9.581, 1e-2)
        assert system_level_metrics['25']['rouge-2']['recall'] == pytest.approx(9.259, 1e-2)
        assert system_level_metrics['17']['rouge-2']['recall'] == pytest.approx(8.855, 1e-2)

        # ROUGE/rouge2_B.jk.m.avg
        assert system_level_metrics['E']['rouge-2_jk']['recall'] == pytest.approx(11.474, 1e-2)
        assert system_level_metrics['H']['rouge-2_jk']['recall'] == pytest.approx(10.865, 1e-1)
        assert system_level_metrics['43']['rouge-2_jk']['recall'] == pytest.approx(9.589, 1e-2)

        # ROUGE/rougeSU4_B.m.avg
        assert system_level_metrics['43']['rouge-su4']['recall'] == pytest.approx(13.080, 1e-2)
        assert system_level_metrics['24']['rouge-su4']['recall'] == pytest.approx(12.803, 1e-2)
        assert system_level_metrics['17']['rouge-su4']['recall'] == pytest.approx(12.792, 1e-2)

        # ROUGE/rougeSU4_B.jk.m.avg
        assert system_level_metrics['E']['rouge-su4_jk']['recall'] == pytest.approx(14.941, 1e-2)
        assert system_level_metrics['D']['rouge-su4_jk']['recall'] == pytest.approx(14.368, 1e-2)
        assert system_level_metrics['43']['rouge-su4_jk']['recall'] == pytest.approx(13.086, 1e-2)

        # manual/manual.model.B.avg
        assert system_level_metrics['A']['num_scus_jk'] == pytest.approx(6.682, 1e-2)
        assert system_level_metrics['B']['num_scus_jk'] == pytest.approx(5.409, 1e-2)
        assert system_level_metrics['C']['num_scus_jk'] == pytest.approx(5.864, 1e-2)

        assert system_level_metrics['A']['modified_pyramid_score_jk'] == pytest.approx(0.663, 1e-2)
        assert system_level_metrics['B']['modified_pyramid_score_jk'] == pytest.approx(0.554, 1e-2)
        assert system_level_metrics['C']['modified_pyramid_score_jk'] == pytest.approx(0.565, 1e-2)

        assert system_level_metrics['A']['linguistic_quality'] == pytest.approx(4.909, 1e-2)
        assert system_level_metrics['B']['linguistic_quality'] == pytest.approx(4.909, 1e-2)
        assert system_level_metrics['C']['linguistic_quality'] == pytest.approx(4.955, 1e-2)

        assert system_level_metrics['A']['overall_responsiveness'] == pytest.approx(4.773, 1e-2)
        assert system_level_metrics['B']['overall_responsiveness'] == pytest.approx(4.500, 1e-2)
        assert system_level_metrics['C']['overall_responsiveness'] == pytest.approx(4.682, 1e-2)

        # manual/manual.peer.B.avg
        assert system_level_metrics['1']['modified_pyramid_score'] == pytest.approx(0.237, 1e-2)
        assert system_level_metrics['2']['modified_pyramid_score'] == pytest.approx(0.284, 1e-2)
        assert system_level_metrics['3']['modified_pyramid_score'] == pytest.approx(0.327, 1e-2)

        assert system_level_metrics['1']['num_scus'] == pytest.approx(2.636, 1e-2)
        assert system_level_metrics['2']['num_scus'] == pytest.approx(3.136, 1e-2)
        assert system_level_metrics['3']['num_scus'] == pytest.approx(3.682, 1e-2)

        assert system_level_metrics['1']['num_repetitions'] == pytest.approx(0.364, 1e-2)
        assert system_level_metrics['2']['num_repetitions'] == pytest.approx(0.568, 1e-2)
        assert system_level_metrics['3']['num_repetitions'] == pytest.approx(0.727, 1e-2)

        assert system_level_metrics['1']['modified_pyramid_score_jk'] == pytest.approx(0.234, 1e-2)
        assert system_level_metrics['2']['modified_pyramid_score_jk'] == pytest.approx(0.280, 1e-2)
        assert system_level_metrics['3']['modified_pyramid_score_jk'] == pytest.approx(0.322, 1e-2)

        assert system_level_metrics['1']['linguistic_quality'] == pytest.approx(3.455, 1e-2)
        assert system_level_metrics['2']['linguistic_quality'] == pytest.approx(2.841, 1e-2)
        assert system_level_metrics['3']['linguistic_quality'] == pytest.approx(2.886, 1e-2)

        assert system_level_metrics['1']['overall_responsiveness'] == pytest.approx(2.091, 1e-2)
        assert system_level_metrics['2']['overall_responsiveness'] == pytest.approx(2.114, 1e-2)
        assert system_level_metrics['3']['overall_responsiveness'] == pytest.approx(2.500, 1e-2)

        # BE/simple_B.m.hm.avg
        assert system_level_metrics['43']['rouge-be-hm']['recall'] == pytest.approx(6.473, 1e-2)
        assert system_level_metrics['25']['rouge-be-hm']['recall'] == pytest.approx(5.937, 1e-2)
        assert system_level_metrics['26']['rouge-be-hm']['recall'] == pytest.approx(5.717, 1e-1)

        # BE/simplejk_B.m.hm.avg
        assert system_level_metrics['E']['rouge-be-hm_jk']['recall'] == pytest.approx(7.970, 1e-2)
        assert system_level_metrics['D']['rouge-be-hm_jk']['recall'] == pytest.approx(7.341, 1e-1)
        assert system_level_metrics['43']['rouge-be-hm_jk']['recall'] == pytest.approx(6.480, 1e-1)

        # aesop_allpeers_B
        assert system_level_metrics['B']['aesop']['2'] == pytest.approx(0.1278890909, 1e-2)
        assert system_level_metrics['E']['aesop']['4'] == pytest.approx(0.4831818182, 1e-2)
        assert system_level_metrics['6']['aesop']['8'] == pytest.approx(1.003988068, 1e-2)



from typing import List, Tuple

from sacrerouge.common.util import download_file_from_google_drive
from sacrerouge.io import JsonlReader, JsonlWriter


def _download_data(output_dir: str, force: bool) -> None:
    download_file_from_google_drive('1kUjSRXzKnTYdJ732BkKVLg3CFxDKo25u', f'{output_dir}/train.jsonl.gz', force=force)
    download_file_from_google_drive('1_kHTZ32jazTbXaFRg0vBeIsVcpI7CTmy', f'{output_dir}/val.jsonl.gz', force=force)
    download_file_from_google_drive('1qsd5pOCpeSXsaqNobXCrcAzhcjtG1wA1', f'{output_dir}/test.jsonl.gz', force=force)


def _load_instances(input_file: str) -> Tuple[List, int]:
    instances = []
    num_empty = 0
    with JsonlReader(input_file) as f:
        for instance in f:
            instance_id = str(instance['id'])
            date = instance['date']
            reference_urls = set(instance['reference_urls'])
            summary = instance['summary']
            documents = []
            for article in instance['articles']:
                document_id = article['id']
                title = article['title']
                text = article['text'].strip()
                url = article['url']
                origin = article['origin']
                time = article['time']

                if len(text) == 0:
                    num_empty += 1
                    continue

                if origin == 'WCEP':
                    assert url in reference_urls

                documents.append({
                    'document_id': document_id,
                    'title': title,
                    'url': url,
                    'time': time,
                    'origin': origin,
                    'text': text
                })
            assert len(documents) > 0

            instances.append({
                'instance_id': instance_id,
                'date': date,
                'documents': documents,
                'summary': {'text': summary}
            })
    return instances, num_empty


def _count_num_documents(instances: List) -> int:
    return sum(len(instance['documents']) for instance in instances)


def _remove_duplicates(instances: List) -> int:
    num_removed = 0
    for instance in instances:
        url_to_origin = {}
        url_to_document = {}
        for document in instance['documents']:
            url = document['url']
            origin = document['origin']
            if url in url_to_origin:
                if origin == 'WCEP' and url_to_origin[url] != 'WCEP':
                    # We prefer the WCEP url not the CommonCrawl url
                    url_to_origin[url] = origin
                    url_to_document[url] = document
                else:
                    # Otherwise, we just leave what is already there
                    pass
                # This is a duplicate, whether we removed the one we already
                # found or this one
                num_removed += 1
            else:
                url_to_origin[url] = origin
                url_to_document[url] = document

        instance['documents'] = list(url_to_document.values())
    return num_removed


def _save(instances: List, output_file: str) -> None:
    with JsonlWriter(output_file) as out:
        for instance in instances:
            out.write(instance)


def setup(output_dir: str, force: bool) -> None:
    _download_data(f'{output_dir}/raw', force)

    train, train_num_empty = _load_instances(f'{output_dir}/raw/train.jsonl.gz')
    valid, valid_num_empty = _load_instances(f'{output_dir}/raw/val.jsonl.gz')
    test, test_num_empty = _load_instances(f'{output_dir}/raw/test.jsonl.gz')

    train_num_duplicates = _remove_duplicates(train)
    valid_num_duplicates = _remove_duplicates(valid)
    test_num_duplicates = _remove_duplicates(test)

    print('Number of empty documents removed')
    print('Train', train_num_empty)
    print('Valid', valid_num_empty)
    print('Test', test_num_empty)
    print()

    print('Number of duplicate source documents removed')
    print('Train', train_num_duplicates)
    print('Valid', valid_num_duplicates)
    print('Test', test_num_duplicates)

    _save(train, f'{output_dir}/train.jsonl.gz')
    _save(valid, f'{output_dir}/valid.jsonl.gz')
    _save(test, f'{output_dir}/test.jsonl.gz')


